@@ -45,7 +45,12 @@ modin.utils._make_api_url = _saving_make_api_url
 
 import modin  # noqa: E402
 import modin.config  # noqa: E402
-from modin.config import IsExperimental, TestRayClient  # noqa: E402
+from modin.config import (  # noqa: E402
+    NPartitions,
+    MinPartitionSize,
+    IsExperimental,
+    TestRayClient,
+)
 import uuid  # noqa: E402
 
 from modin.core.storage_formats import (  # noqa: E402
@@ -424,7 +429,6 @@ def create_fixture(file_type):
 
 
 for file_type in ("json", "html", "excel", "feather", "stata", "hdf", "pickle", "fwf"):
-
     fixture = create_fixture(file_type)
     fixture.__name__ = f"make_{file_type}_file"
     globals()[fixture.__name__] = pytest.fixture(fixture)
@@ -536,6 +540,22 @@ def get_generated_doc_urls():
     return lambda: _generated_doc_urls
 
 
+@pytest.fixture
+def set_num_partitions(request):
+    old_num_partitions = NPartitions.get()
+    NPartitions.put(request.param)
+    yield
+    NPartitions.put(old_num_partitions)
+
+
+@pytest.fixture
+def set_min_partition_size(request):
+    old_min_partition_size = MinPartitionSize.get()
+    MinPartitionSize.put(request.param)
+    yield
+    MinPartitionSize.put(old_min_partition_size)
+
+
 ray_client_server = None
 
 
