@@ -13,17 +13,34 @@
 
 """Module houses builder class for GroupByReduce operator."""
 
-from collections.abc import Container
 import pandas
 
 from .tree_reduce import TreeReduce
-from .default2pandas.groupby import GroupBy
-from modin.utils import try_cast_to_pandas, hashable, MODIN_UNNAMED_SERIES_LABEL
+from .default2pandas.groupby import GroupBy, GroupByDefault
+from modin.utils import hashable, MODIN_UNNAMED_SERIES_LABEL
 from modin.error_message import ErrorMessage
 
 
 class GroupByReduce(TreeReduce):
-    """Builder class for GroupBy aggregation functions."""
+    """
+    Builder class for GroupBy aggregation functions.
+
+    Attributes
+    ----------
+    ID_LEVEL_NAME : str
+        It's supposed that implementations may produce multiple temporary
+        columns per one source column in an intermediate phase. In order
+        for these columns to be processed accordingly at the Reduce phase,
+        an implementation must store unique names for such temporary
+        columns in the ``ID_LEVEL_NAME`` level. Duplicated names are not allowed.
+    _GROUPBY_REDUCE_IMPL_FLAG : str
+        Attribute indicating that a callable should be treated as an
+        implementation for one of the TreeReduce phases rather than an
+        arbitrary aggregation. Note: this attribute should be considered private.
+    """
+
+    ID_LEVEL_NAME = "__ID_LEVEL_NAME__"
+    _GROUPBY_REDUCE_IMPL_FLAG = "__groupby_reduce_impl_func__"
 
     @classmethod
     def register(cls, map_func, reduce_func=None, **call_kwds):
@@ -34,13 +51,11 @@ class GroupByReduce(TreeReduce):
 
         Parameters
         ----------
-        map_func : str, dict or callable(pandas.DataFrameGroupBy) -> pandas.DataFrame
-            If `str` this parameter will be treated as a function name to register,
-            so `map_func` and `reduce_func` will be grabbed from `groupby_reduce_functions`.
-            If dict or callable then this will be treated as a function to apply to the `GroupByObject`
-            at the map phase.
-        reduce_func : str, dict or callable(pandas.DataFrameGroupBy) -> pandas.DataFrame, optional
-            Function to apply to the `GroupByObject` at the reduce phase. If not specified
+        map_func : str, dict or callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame
+            Function to apply to the `GroupByObject` at the map phase. If ``str`` was passed it will
+            be treated as a DataFrameGroupBy's method name.
+        reduce_func : str, dict or callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame, optional
+            Function to apply to the ``DataFrameGroupBy`` at the reduce phase. If not specified
             will be set the same as 'map_func'.
         **call_kwds : kwargs
             Kwargs that will be passed to the returned function.
@@ -51,14 +66,17 @@ class GroupByReduce(TreeReduce):
             Function that takes query compiler and executes GroupBy aggregation
             with TreeReduce algorithm.
         """
-        if isinstance(map_func, str):
-
-            def build_fn(name):
-                return lambda df, *args, **kwargs: getattr(df, name)(*args, **kwargs)
-
-            map_func, reduce_func = map(build_fn, groupby_reduce_functions[map_func])
         if reduce_func is None:
             reduce_func = map_func
+
+        def build_fn(name):
+            return lambda df, *args, **kwargs: getattr(df, name)(*args, **kwargs)
+
+        if isinstance(map_func, str):
+            map_func = build_fn(map_func)
+        if isinstance(reduce_func, str):
+            reduce_func = build_fn(reduce_func)
+
         assert not (
             isinstance(map_func, dict) ^ isinstance(reduce_func, dict)
         ) and not (
@@ -69,6 +87,21 @@ class GroupByReduce(TreeReduce):
             *args, map_func=map_func, reduce_func=reduce_func, **kwargs, **call_kwds
         )
 
+    @classmethod
+    def register_implementation(cls, map_func, reduce_func):
+        """
+        Register callables to be recognized as an implementations of tree-reduce phases.
+
+        Parameters
+        ----------
+        map_func : callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame
+            Callable to register.
+        reduce_func : callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame
+            Callable to register.
+        """
+        setattr(map_func, cls._GROUPBY_REDUCE_IMPL_FLAG, True)
+        setattr(reduce_func, cls._GROUPBY_REDUCE_IMPL_FLAG, True)
+
     @classmethod
     def map(
         cls,
@@ -80,6 +113,7 @@ class GroupByReduce(TreeReduce):
         agg_kwargs,
         other=None,
         by=None,
+        drop=False,
     ):
         """
         Execute Map phase of GroupByReduce.
@@ -108,6 +142,8 @@ class GroupByReduce(TreeReduce):
         by : level index name or list of such labels, optional
             Index levels, that is used to determine groups.
             If not specified, `other` parameter is used.
+        drop : bool, default: False
+            Indicates whether or not by-data came from the `self` frame.
 
         Returns
         -------
@@ -122,12 +158,22 @@ class GroupByReduce(TreeReduce):
         # We have to filter func-dict BEFORE inserting broadcasted 'by' columns
         # to avoid multiple aggregation results for 'by' cols in case they're
         # present in the func-dict:
-        apply_func = cls.try_filter_dict(map_func, df)
+        apply_func = cls.get_callable(
+            map_func,
+            df,
+            # We won't be able to preserve the order as the Map phase would likely
+            # produce some temporary columns that won't fit into the original
+            # aggregation order. It doesn't matter much as we restore the original
+            # order at the Reduce phase.
+            preserve_aggregation_order=False,
+        )
         if other is not None:
-            # Other is a broadcasted partition that represents 'by' columns
-            # Concatenate it with 'df' to group on its columns names
-            other = other.squeeze(axis=axis ^ 1)
-            if isinstance(other, pandas.DataFrame):
+            # Other is a broadcasted partition that represents 'by' data to group on.
+            # If 'drop' then the 'by' data came from the 'self' frame, thus
+            # inserting missed columns to the partition to group on them.
+            if drop or isinstance(
+                other := other.squeeze(axis=axis ^ 1), pandas.DataFrame
+            ):
                 df = pandas.concat(
                     [df] + [other[[o for o in other if o not in df]]],
                     axis=1,
@@ -192,12 +238,6 @@ class GroupByReduce(TreeReduce):
         # there is a bug in pandas with intersection that forces us to do so:
         # https://github.com/pandas-dev/pandas/issues/39699
         by_part = pandas.Index(df.index.names)
-        if drop:
-            to_drop = df.columns.intersection(by_part)
-            if isinstance(reduce_func, dict):
-                to_drop = to_drop.difference(reduce_func.keys())
-            if len(to_drop) > 0:
-                df.drop(columns=by_part, errors="ignore", inplace=True)
 
         groupby_kwargs = groupby_kwargs.copy()
         as_index = groupby_kwargs["as_index"]
@@ -209,7 +249,7 @@ class GroupByReduce(TreeReduce):
         # we want to group on these levels
         groupby_kwargs["level"] = list(range(len(df.index.names)))
 
-        apply_func = cls.try_filter_dict(reduce_func, df)
+        apply_func = cls.get_callable(reduce_func, df)
         result = apply_func(
             df.groupby(axis=axis, **groupby_kwargs), *agg_args, **agg_kwargs
         )
@@ -284,31 +324,41 @@ class GroupByReduce(TreeReduce):
         The same type as `query_compiler`
             QueryCompiler which carries the result of GroupBy aggregation.
         """
+        is_unsupported_axis = axis != 0
+        # Defaulting to pandas in case of an empty frame as we can't process it properly.
+        # Higher API level won't pass empty data here unless the frame has delayed
+        # computations. So we apparently lose some laziness here (due to index access)
+        # because of the inability to process empty groupby natively.
+        is_empty_data = (
+            len(query_compiler.columns) == 0 or len(query_compiler.index) == 0
+        )
+        is_grouping_using_by_arg = (
+            groupby_kwargs.get("level", None) is None and by is not None
+        )
+        is_unsupported_by_arg = isinstance(by, pandas.Grouper) or (
+            not hashable(by) and not isinstance(by, type(query_compiler))
+        )
+
         if (
-            axis != 0
-            or groupby_kwargs.get("level", None) is None
-            and (
-                not (isinstance(by, (type(query_compiler))) or hashable(by))
-                or isinstance(by, pandas.Grouper)
-            )
+            is_unsupported_axis
+            or is_empty_data
+            or (is_grouping_using_by_arg and is_unsupported_by_arg)
         ):
-            by = try_cast_to_pandas(by, squeeze=True)
-            # Since 'by' may be a 2D query compiler holding columns to group by,
-            # to_pandas will also produce a pandas DataFrame containing them.
-            # So splitting 2D 'by' into a list of 1D Series using 'GroupBy.validate_by':
-            by = GroupBy.validate_by(by)
             if default_to_pandas_func is None:
                 default_to_pandas_func = (
                     (lambda grp: grp.agg(map_func))
                     if isinstance(map_func, dict)
                     else map_func
                 )
-            return query_compiler.default_to_pandas(
-                lambda df: default_to_pandas_func(
-                    df.groupby(by=by, axis=axis, **groupby_kwargs),
-                    *agg_args,
-                    **agg_kwargs,
-                )
+            default_to_pandas_func = GroupByDefault.register(default_to_pandas_func)
+            return default_to_pandas_func(
+                query_compiler,
+                by=by,
+                axis=axis,
+                groupby_kwargs=groupby_kwargs,
+                agg_args=agg_args,
+                agg_kwargs=agg_kwargs,
+                drop=drop,
             )
 
         # The bug only occurs in the case of Categorical 'by', so we might want to check whether any of
@@ -354,8 +404,8 @@ class GroupByReduce(TreeReduce):
             result.index.name = None
         return result
 
-    @staticmethod
-    def try_filter_dict(agg_func, df):
+    @classmethod
+    def get_callable(cls, agg_func, df, preserve_aggregation_order=True):
         """
         Build aggregation function to apply to each group at this particular partition.
 
@@ -368,16 +418,215 @@ class GroupByReduce(TreeReduce):
             Aggregation function.
         df : pandas.DataFrame
             Serialized partition which contains available columns.
+        preserve_aggregation_order : bool, default: True
+            Whether to manually restore the order of columns for the result specified
+            by the `agg_func` keys (only makes sense when `agg_func` is a dictionary).
 
         Returns
         -------
-        Callable
+        callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame
             Aggregation function that can be safely applied to this particular partition.
         """
         if not isinstance(agg_func, dict):
             return agg_func
-        partition_dict = {k: v for k, v in agg_func.items() if k in df.columns}
-        return lambda grp: grp.agg(partition_dict)
+
+        grp_has_id_level = df.columns.names[0] == cls.ID_LEVEL_NAME
+        # The 'id' level prevents us from a lookup for the original
+        # partition's columns. So dropping the level.
+        partition_columns = frozenset(
+            df.columns.droplevel(0) if grp_has_id_level else df.columns
+        )
+
+        partition_dict = {k: v for k, v in agg_func.items() if k in partition_columns}
+        return cls._build_callable_for_dict(
+            partition_dict, preserve_aggregation_order, grp_has_id_level
+        )
+
+    @classmethod
+    def _build_callable_for_dict(
+        cls, agg_dict, preserve_aggregation_order=True, grp_has_id_level=False
+    ):
+        """
+        Build callable for an aggregation dictionary.
+
+        Parameters
+        ----------
+        agg_dict : dict
+            Aggregation dictionary.
+        preserve_aggregation_order : bool, default: True
+            Whether to manually restore the order of columns for the result specified
+            by the `agg_func` keys (only makes sense when `agg_func` is a dictionary).
+        grp_has_id_level : bool, default: False
+            Whether the frame we're grouping on has intermediate columns
+            (see ``cls.ID_LEVEL_NAME``).
+
+        Returns
+        -------
+        callable(pandas.core.groupby.DataFrameGroupBy) -> pandas.DataFrame
+        """
+        # We have to keep this import away from the module level to avoid circular import
+        from modin.pandas.utils import walk_aggregation_dict
+
+        # We now filter aggregation functions into those that could be applied natively
+        # using pandas (pandas_grp_obj.agg(**native_aggs)) and those that require
+        # special treatment (custom_aggs).
+        custom_aggs = {}
+        native_aggs = {}
+
+        result_columns = []
+        for col, func, func_name, col_renaming_required in walk_aggregation_dict(
+            agg_dict
+        ):
+            # Filter dictionary
+            dict_to_add = (
+                custom_aggs if cls.is_registered_implementation(func) else native_aggs
+            )
+
+            new_value = func if func_name is None else (func_name, func)
+            old_value = dict_to_add.get(col, None)
+
+            if old_value is not None:
+                ErrorMessage.catch_bugs_and_request_email(
+                    failure_condition=not isinstance(old_value, list),
+                    extra_log="Expected for all aggregation values to be a list when at least "
+                    + f"one column has multiple aggregations. Got: {old_value} {type(old_value)}",
+                )
+                old_value.append(new_value)
+            else:
+                # Pandas knows that it has to modify the resulting columns if it meets
+                # a function wrapped into a list. Renaming is required if either a new
+                # column name was explicitly specified, or multiple functions were
+                # specified per one column, or if any other column in the aggregation
+                # is going to be renamed.
+                dict_to_add[col] = [new_value] if col_renaming_required else new_value
+
+            # Construct resulting columns
+            if col_renaming_required:
+                func_name = str(func) if func_name is None else func_name
+                result_columns.append(
+                    (*(col if isinstance(col, tuple) else (col,)), func_name)
+                )
+            else:
+                result_columns.append(col)
+
+        result_columns = pandas.Index(result_columns)
+
+        def aggregate_on_dict(grp_obj, *args, **kwargs):
+            """Aggregate the passed groupby object."""
+            if len(native_aggs) == 0:
+                native_agg_res = None
+            elif grp_has_id_level:
+                # Adding the 'id' level to the aggregation keys so they match `grp_obj` columns
+                native_aggs_modified = {
+                    (
+                        cls.ID_LEVEL_NAME,
+                        *(key if isinstance(key, tuple) else (key,)),
+                    ): value
+                    for key, value in native_aggs.items()
+                }
+                native_agg_res = grp_obj.agg(native_aggs_modified)
+                # Dropping the 'id' level from the resulted frame
+                native_agg_res.columns = native_agg_res.columns.droplevel(0)
+            else:
+                native_agg_res = grp_obj.agg(native_aggs)
+
+            custom_results = []
+            insert_id_levels = False
+
+            for col, func, func_name, col_renaming_required in walk_aggregation_dict(
+                custom_aggs
+            ):
+                if grp_has_id_level:
+                    cols_without_ids = grp_obj.obj.columns.droplevel(0)
+                    if isinstance(cols_without_ids, pandas.MultiIndex):
+                        # We may have multiple columns matching the `col` in
+                        # a MultiIndex case, that's why use `.get_locs` here
+                        col_pos = cols_without_ids.get_locs(col)
+                    else:
+                        # `pandas.Index` doesn't have `.get_locs` method
+                        col_pos = cols_without_ids.get_loc(col)
+                    agg_key = grp_obj.obj.columns[col_pos]
+                else:
+                    agg_key = [col]
+
+                result = func(grp_obj[agg_key])
+                # The `func` may have discarded an ID-level if there were any.
+                # So checking for this again.
+                result_has_id_level = result.columns.names[0] == cls.ID_LEVEL_NAME
+                insert_id_levels |= result_has_id_level
+
+                if col_renaming_required:
+                    func_name = str(func) if func_name is None else func_name
+                    if result_has_id_level:
+                        result.columns = pandas.MultiIndex.from_tuples(
+                            [
+                                # `old_col[0]` stores values from the 'id'
+                                # level, the ones we want to preserve here
+                                (old_col[0], col, func_name)
+                                for old_col in result.columns
+                            ],
+                            names=[
+                                result.columns.names[0],
+                                result.columns.names[1],
+                                None,
+                            ],
+                        )
+                    else:
+                        result.columns = pandas.MultiIndex.from_tuples(
+                            [(col, func_name)] * len(result.columns),
+                            names=[result.columns.names[0], None],
+                        )
+
+                custom_results.append(result)
+
+            if insert_id_levels:
+                # As long as any `result` has an id-level we have to insert the level
+                # into every `result` so the number of levels matches
+                for idx, ext_result in enumerate(custom_results):
+                    if ext_result.columns.names[0] != cls.ID_LEVEL_NAME:
+                        custom_results[idx] = pandas.concat(
+                            [ext_result],
+                            keys=[cls.ID_LEVEL_NAME],
+                            names=[cls.ID_LEVEL_NAME],
+                            axis=1,
+                            copy=False,
+                        )
+
+                if native_agg_res is not None:
+                    native_agg_res = pandas.concat(
+                        [native_agg_res],
+                        keys=[cls.ID_LEVEL_NAME],
+                        names=[cls.ID_LEVEL_NAME],
+                        axis=1,
+                        copy=False,
+                    )
+
+            native_res_part = [] if native_agg_res is None else [native_agg_res]
+            result = pandas.concat(
+                [*native_res_part, *custom_results], axis=1, copy=False
+            )
+
+            # The order is naturally preserved if there's no custom aggregations
+            if preserve_aggregation_order and len(custom_aggs):
+                result = result.reindex(result_columns, axis=1)
+            return result
+
+        return aggregate_on_dict
+
+    @classmethod
+    def is_registered_implementation(cls, func):
+        """
+        Check whether the passed `func` was registered as a TreeReduce implementation.
+
+        Parameters
+        ----------
+        func : callable
+
+        Returns
+        -------
+        bool
+        """
+        return callable(func) and hasattr(func, cls._GROUPBY_REDUCE_IMPL_FLAG)
 
     @classmethod
     def build_map_reduce_functions(
@@ -439,6 +688,7 @@ class GroupByReduce(TreeReduce):
                     map_func=map_func,
                     agg_args=agg_args,
                     agg_kwargs=agg_kwargs,
+                    drop=drop,
                     **kwargs,
                 )
 
@@ -473,73 +723,3 @@ class GroupByReduce(TreeReduce):
             return result
 
         return _map, _reduce
-
-
-# This dict is a map for function names and their equivalents in TreeReduce
-groupby_reduce_functions = {
-    "all": ("all", "all"),
-    "any": ("any", "any"),
-    "count": ("count", "sum"),
-    "max": ("max", "max"),
-    "min": ("min", "min"),
-    "prod": ("prod", "prod"),
-    "size": ("size", "sum"),
-    "sum": ("sum", "sum"),
-}
-
-
-def _is_reduce_function_with_depth(fn, depth: int = 0):
-    """
-    Check whether all functions defined by `fn` are groupby reductions.
-
-    If true, all functions defined by `fn` can be implemented with TreeReduce.
-    This is a recursive helper function for is_reduce_function.
-
-    Parameters
-    ----------
-    fn : Any
-        Function to test.
-    depth : int, default: 0
-        How many nested containers we are within for this check.
-            - if it's 0, then we're outside of any container, and `fn` could be
-              either function name or container of function names/renamers.
-            - if it's 1, then we're inside container of function
-              names/renamers.`fn` must be either function name or renamer.
-              renamer is some container which length == 2, where the first
-              element is the new column name and the second is the function
-              name.
-
-    Returns
-    -------
-    bool
-        Whether all functions defined by `fn` are reductions.
-    """
-    if not isinstance(fn, str) and isinstance(fn, Container):
-        assert depth == 0 or (
-            depth > 0 and len(fn) == 2
-        ), f"Got the renamer with incorrect length, expected 2 got {len(fn)}."
-        return (
-            all(_is_reduce_function_with_depth(f, depth + 1) for f in fn)
-            if depth == 0
-            else _is_reduce_function_with_depth(fn[1], depth + 1)
-        )
-    return isinstance(fn, str) and fn in groupby_reduce_functions
-
-
-def is_reduce_function(fn):
-    """
-    Check whether all functions defined by `fn` are groupby reductions.
-
-    If true, all functions defined by `fn` can be implemented with TreeReduce.
-
-    Parameters
-    ----------
-    fn : Any
-        Function to test.
-
-    Returns
-    -------
-    bool
-        Whether all functions defined by `fn` are reductions.
-    """
-    return _is_reduce_function_with_depth(fn, depth=0)
