@@ -14,8 +14,9 @@
 """Module houses `ParquetDispatcher` class, that is used for reading `.parquet` files."""
 
 import os
-
+import re
 import json
+
 import fsspec
 from fsspec.core import url_to_fs
 from fsspec.spec import AbstractBufferedFile
@@ -27,7 +28,7 @@ from modin.config import NPartitions
 
 
 from modin.core.io.column_stores.column_store_dispatcher import ColumnStoreDispatcher
-from modin.utils import import_optional_dependency, _inherit_docstrings
+from modin.utils import _inherit_docstrings
 
 
 class ColumnStoreDataset:
@@ -157,6 +158,7 @@ class ColumnStoreDataset:
         # Older versions of fsspec doesn't support unstrip_protocol(). It
         # was only added relatively recently:
         # https://github.com/fsspec/filesystem_spec/pull/828
+
         def _unstrip_protocol(protocol, path):
             protos = (protocol,) if isinstance(protocol, str) else protocol
             for protocol in protos:
@@ -214,7 +216,12 @@ class PyArrowDataset(ColumnStoreDataset):
     @property
     def files(self):
         if self._files is None:
-            self._files = self._get_files(self.dataset.files)
+            try:
+                files = self.dataset.files
+            except AttributeError:
+                # compatibility at least with 3.0.0 <= pyarrow < 8.0.0
+                files = self.dataset._dataset.files
+            self._files = self._get_files(files)
         return self._files
 
     def to_pandas_dataframe(
@@ -292,6 +299,8 @@ class FastParquetDataset(ColumnStoreDataset):
 class ParquetDispatcher(ColumnStoreDispatcher):
     """Class handles utils for reading `.parquet` files."""
 
+    index_regex = re.compile(r"__index_level_\d+__")
+
     @classmethod
     def get_dataset(cls, path, engine, storage_options):
         """
@@ -603,24 +612,35 @@ class ParquetDispatcher(ColumnStoreDispatcher):
         ParquetFile API is used. Please refer to the documentation here
         https://arrow.apache.org/docs/python/parquet.html
         """
-        import_optional_dependency(
-            "pyarrow",
-            "pyarrow is required to read parquet files.",
-        )
-        from modin.pandas.io import PQ_INDEX_REGEX
-
-        if isinstance(path, str) and os.path.isdir(path):
+        if isinstance(path, list):
+            # TODO(https://github.com/modin-project/modin/issues/5723): read all
+            # files in parallel.
+            compilers: list[cls.query_compiler_cls] = [
+                cls._read(p, engine, columns, **kwargs) for p in path
+            ]
+            return compilers[0].concat(axis=0, other=compilers[1:], ignore_index=True)
+        if isinstance(path, str):
+            if os.path.isdir(path):
+                path_generator = os.walk(path)
+            else:
+                storage_options = kwargs.get("storage_options")
+                if storage_options is not None:
+                    fs, fs_path = url_to_fs(path, **storage_options)
+                else:
+                    fs, fs_path = url_to_fs(path)
+                path_generator = fs.walk(fs_path)
             partitioned_columns = set()
             # We do a tree walk of the path directory because partitioned
             # parquet directories have a unique column at each directory level.
             # Thus, we can use os.walk(), which does a dfs search, to walk
             # through the different columns that the data is partitioned on
-            for (_, dir_names, files) in os.walk(path):
+            for _, dir_names, files in path_generator:
                 if dir_names:
                     partitioned_columns.add(dir_names[0].split("=")[0])
                 if files:
                     # Metadata files, git files, .DSStore
-                    if files[0][0] == ".":
+                    # TODO: fix conditional for column partitioning, see issue #4637
+                    if len(files[0]) > 0 and files[0][0] == ".":
                         continue
                     break
             partitioned_columns = list(partitioned_columns)
@@ -644,7 +664,7 @@ class ParquetDispatcher(ColumnStoreDispatcher):
         columns = [
             c
             for c in column_names
-            if c not in index_columns and not PQ_INDEX_REGEX.match(c)
+            if c not in index_columns and not cls.index_regex.match(c)
         ]
 
         return cls.build_query_compiler(dataset, columns, index_columns, **kwargs)
