@@ -28,16 +28,23 @@ from modin.error_message import ErrorMessage
 from modin.core.storage_formats.base.query_compiler import BaseQueryCompiler
 from modin.utils import _inherit_docstrings
 
-from modin._compat.core.base_io import (
-    BaseIOCompat,
-    _doc_default_io_method,
-    _doc_returns_qc,
-    _doc_returns_qc_or_parser,
-)
-from modin._compat.core.pandas_common import pandas_to_csv
+_doc_default_io_method = """
+{summary} using pandas.
+For parameters description please refer to pandas API.
+
+Returns
+-------
+{returns}
+"""
+
+_doc_returns_qc = """BaseQueryCompiler
+    QueryCompiler with read data."""
+
+_doc_returns_qc_or_parser = """BaseQueryCompiler or TextParser
+    QueryCompiler or TextParser with read data."""
 
 
-class BaseIO(BaseIOCompat):
+class BaseIO:
     """Class for basic utils and default implementation of IO functions."""
 
     query_compiler_cls: BaseQueryCompiler = None
@@ -115,7 +122,7 @@ class BaseIO(BaseIOCompat):
         summary="Load a parquet object from the file path, returning a query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_parquet(cls, **kwargs):  # noqa: PR01
+    def read_parquet(cls, **kwargs):  # noqa: PR01
         ErrorMessage.default_to_pandas("`read_parquet`")
         return cls.from_pandas(
             pandas.read_parquet(
@@ -130,39 +137,20 @@ class BaseIO(BaseIOCompat):
         summary="Read a comma-separated values (CSV) file into query compiler",
         returns=_doc_returns_qc_or_parser,
     )
-    def _read_csv(
+    def read_csv(
         cls,
         filepath_or_buffer,
         **kwargs,
     ):  # noqa: PR01
         ErrorMessage.default_to_pandas("`read_csv`")
-        return cls._read(filepath_or_buffer=filepath_or_buffer, **kwargs)
-
-    @classmethod
-    def _read(cls, **kwargs):
-        """
-        Read csv file into query compiler.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            `read_csv` function kwargs including `filepath_or_buffer` parameter.
-
-        Returns
-        -------
-        BaseQueryCompiler
-            QueryCompiler with read data.
-        """
-        pd_obj = pandas.read_csv(**kwargs)
+        pd_obj = pandas.read_csv(filepath_or_buffer, **kwargs)
         if isinstance(pd_obj, pandas.DataFrame):
             return cls.from_pandas(pd_obj)
         if isinstance(pd_obj, pandas.io.parsers.TextFileReader):
             # Overwriting the read method should return a Modin DataFrame for calls
             # to __next__ and get_chunk
             pd_read = pd_obj.read
-            pd_obj.read = lambda *args, **kwargs: cls.from_pandas(
-                pd_read(*args, **kwargs)
-            )
+            pd_obj.read = lambda *args, **kw: cls.from_pandas(pd_read(*args, **kw))
         return pd_obj
 
     @classmethod
@@ -172,7 +160,7 @@ class BaseIO(BaseIOCompat):
         summary="Convert a JSON string to query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_json(
+    def read_json(
         cls,
         **kwargs,
     ):  # noqa: PR01
@@ -290,67 +278,9 @@ class BaseIO(BaseIOCompat):
         returns="""BaseQueryCompiler or dict/OrderedDict :
     QueryCompiler or OrderedDict/dict with read data.""",
     )
-    def read_excel(
-        cls,
-        io,
-        sheet_name=0,
-        header=0,
-        names=None,
-        index_col=None,
-        usecols=None,
-        squeeze=False,
-        dtype=None,
-        engine=None,
-        converters=None,
-        true_values=None,
-        false_values=None,
-        skiprows=None,
-        nrows=None,
-        na_values=None,
-        keep_default_na=True,
-        verbose=False,
-        parse_dates=False,
-        date_parser=None,
-        thousands=None,
-        comment=None,
-        skip_footer=0,
-        skipfooter=0,
-        convert_float=True,
-        mangle_dupe_cols=True,
-        na_filter=True,
-        **kwds,
-    ):  # noqa: PR01
-        if skip_footer != 0:
-            skipfooter = skip_footer
+    def read_excel(cls, **kwargs):  # noqa: PR01
         ErrorMessage.default_to_pandas("`read_excel`")
-        intermediate = pandas.read_excel(
-            io,
-            sheet_name=sheet_name,
-            header=header,
-            names=names,
-            index_col=index_col,
-            usecols=usecols,
-            squeeze=squeeze,
-            dtype=dtype,
-            engine=engine,
-            converters=converters,
-            true_values=true_values,
-            false_values=false_values,
-            skiprows=skiprows,
-            nrows=nrows,
-            na_values=na_values,
-            keep_default_na=keep_default_na,
-            verbose=verbose,
-            parse_dates=parse_dates,
-            date_parser=date_parser,
-            thousands=thousands,
-            comment=comment,
-            skipfooter=skipfooter,
-            convert_float=convert_float,
-            mangle_dupe_cols=mangle_dupe_cols,
-            na_filter=na_filter,
-            **kwds,
-        )
+        intermediate = pandas.read_excel(**kwargs)
         if isinstance(intermediate, (OrderedDict, dict)):
             parsed = type(intermediate)()
             for key in intermediate.keys():
@@ -411,7 +341,7 @@ class BaseIO(BaseIOCompat):
         summary="Load a feather-format object from the file path into query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_feather(
+    def read_feather(
         cls,
         path,
         **kwargs,
@@ -431,7 +361,7 @@ class BaseIO(BaseIOCompat):
         summary="Read Stata file into query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_stata(
+    def read_stata(
         cls,
         filepath_or_buffer,
         **kwargs,
@@ -476,7 +406,7 @@ class BaseIO(BaseIOCompat):
         summary="Load pickled pandas object (or any object) from file into query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_pickle(
+    def read_pickle(
         cls,
         filepath_or_buffer,
         **kwargs,
@@ -591,7 +521,7 @@ class BaseIO(BaseIOCompat):
         summary="Read SQL query into query compiler",
         returns=_doc_returns_qc,
     )
-    def _read_sql_query(
+    def read_sql_query(
         cls,
         sql,
         con,
@@ -655,7 +585,7 @@ class BaseIO(BaseIOCompat):
     @_inherit_docstrings(
         pandas.DataFrame.to_pickle, apilink="pandas.DataFrame.to_pickle"
     )
-    def _to_pickle(
+    def to_pickle(
         cls,
         obj: Any,
         filepath_or_buffer,
@@ -686,7 +616,7 @@ class BaseIO(BaseIOCompat):
         if isinstance(obj, BaseQueryCompiler):
             obj = obj.to_pandas()
 
-        return pandas_to_csv(obj, **kwargs)
+        return obj.to_csv(**kwargs)
 
     @classmethod
     @_inherit_docstrings(
