@@ -285,7 +285,7 @@ class DFAlgQueryCompiler(BaseQueryCompiler):
         else:
             return self.default_to_pandas(pandas.DataFrame.merge, right, **kwargs)
 
-    def take_2d(self, index=None, columns=None):
+    def take_2d_positional(self, index=None, columns=None):
         return self.__constructor__(
             self._modin_frame.take_2d_labels_or_positional(
                 row_positions=index, col_positions=columns
@@ -555,8 +555,14 @@ class DFAlgQueryCompiler(BaseQueryCompiler):
         )
         return self.__constructor__(new_modin_frame)
 
-    def drop(self, index=None, columns=None):
-        assert index is None, "Only column drop is supported"
+    def drop(self, index=None, columns=None, errors: str = "raise"):
+        if index is not None:
+            raise NotImplementedError("Row drop")
+        if errors != "raise":
+            raise NotImplementedError(
+                "This lazy query compiler will always "
+                + "raise an error on invalid columns."
+            )
         return self.__constructor__(
             self._modin_frame.take_2d_labels_or_positional(
                 row_labels=index, col_labels=self.columns.drop(columns)
@@ -687,9 +693,15 @@ class DFAlgQueryCompiler(BaseQueryCompiler):
             self._modin_frame.reset_index(drop), shape_hint=shape_hint
         )
 
-    def astype(self, col_dtypes, **kwargs):
+    def astype(self, col_dtypes, errors: str = "raise"):
+        if errors != "raise":
+            raise NotImplementedError(
+                "This lazy query compiler will always "
+                + "raise an error on invalid type keys."
+            )
         return self.__constructor__(
-            self._modin_frame.astype(col_dtypes), self._shape_hint
+            self._modin_frame.astype(col_dtypes),
+            self._shape_hint,
         )
 
     def setitem(self, axis, key, value):
