@@ -14,29 +14,13 @@
 import pandas
 import warnings
 
-from modin._compat import PandasCompatVersion
+__pandas_version__ = "1.5.3"
 
-if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36:
-    __pandas_version__ = "1.1.5"
-
-    if pandas.__version__ != __pandas_version__:
-        warnings.warn(
-            f"The pandas version installed ({pandas.__version__}) does not match the pandas version"
-            + f" Modin supports ({__pandas_version__}) in Python 3.6 legacy compatibility mode."
-            + " This may cause undesired side effects!"
-        )
-    else:
-        warnings.warn(
-            f"Starting Modin in compatibility mode to support legacy pandas version {__pandas_version__}"
-        )
-elif PandasCompatVersion.CURRENT == PandasCompatVersion.LATEST:
-    __pandas_version__ = "1.5.1"
-
-    if pandas.__version__ != __pandas_version__:
-        warnings.warn(
-            f"The pandas version installed ({pandas.__version__}) does not match the pandas version"
-            + f" Modin supports ({__pandas_version__}). This may cause undesired side effects!"
-        )
+if pandas.__version__ != __pandas_version__:
+    warnings.warn(
+        f"The pandas version installed ({pandas.__version__}) does not match the supported pandas version in"
+        + f" Modin ({__pandas_version__}). This may cause undesired side effects!"
+    )
 
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")
@@ -45,7 +29,6 @@ with warnings.catch_warnings():
         cut,
         factorize,
         test,
-        qcut,
         date_range,
         period_range,
         Index,
@@ -57,7 +40,11 @@ with warnings.catch_warnings():
         Timestamp,
         set_eng_float_format,
         options,
+        describe_option,
         set_option,
+        get_option,
+        reset_option,
+        option_context,
         NaT,
         PeriodIndex,
         Categorical,
@@ -96,25 +83,27 @@ with warnings.catch_warnings():
         NamedAgg,
         NA,
         api,
+        ArrowDtype,
+        Flags,
+        Float32Dtype,
+        Float64Dtype,
+        from_dummies,
     )
 import os
 
-from modin.config import Engine, Parameter
-
-# Set this so that Pandas doesn't try to multithread by itself
-os.environ["OMP_NUM_THREADS"] = "1"
+from modin.config import Parameter
 
 _is_first_update = {}
-_NOINIT_ENGINES = {
-    "Python",
-}  # engines that don't require initialization, useful for unit tests
 
 
 def _update_engine(publisher: Parameter):
-    from modin.config import StorageFormat, CpuCount
+    from modin.config import Engine, StorageFormat, CpuCount
     from modin.config.envvars import IsExperimental
     from modin.config.pubsub import ValueSource
 
+    # Set this so that Pandas doesn't try to multithread by itself
+    os.environ["OMP_NUM_THREADS"] = "1"
+
     sfmt = StorageFormat.get()
 
     if sfmt == "Hdk":
@@ -159,6 +148,11 @@ def _update_engine(publisher: Parameter):
             from modin.core.execution.dask.common import initialize_dask
 
             initialize_dask()
+    elif publisher.get() == "Unidist":
+        if _is_first_update.get("Unidist", True):
+            from modin.core.execution.unidist.common import initialize_unidist
+
+            initialize_unidist()
     elif publisher.get() == "Cloudray":
         from modin.experimental.cloud import get_connection
 
@@ -196,7 +190,7 @@ def _update_engine(publisher: Parameter):
         ), f"Storage format should be 'Hdk' with 'Cloudnative' engine, but provided {sfmt}."
         get_connection().modules["modin"].set_execution("Native", "Hdk")
 
-    elif publisher.get() not in _NOINIT_ENGINES:
+    elif publisher.get() not in Engine.NOINIT_ENGINES:
         raise ImportError("Unrecognized execution engine: {}.".format(publisher.get()))
 
     _is_first_update[publisher.get()] = False
@@ -242,6 +236,7 @@ from .general import (
     notna,
     pivot,
     to_numeric,
+    qcut,
     to_datetime,
     unique,
     value_counts,
@@ -251,11 +246,9 @@ from .general import (
     lreshape,
     wide_to_long,
     to_timedelta,
+    pivot_table,
 )
 
-from modin._compat.pandas_api.namespace import pivot_table
-from modin._compat import PandasCompatVersion
-
 from .plotting import Plotting as plotting
 from modin.utils import show_versions
 
@@ -301,7 +294,11 @@ __all__ = [  # noqa: F405
     "to_timedelta",
     "set_eng_float_format",
     "options",
+    "describe_option",
     "set_option",
+    "get_option",
+    "reset_option",
+    "option_context",
     "CategoricalIndex",
     "Timedelta",
     "Timestamp",
@@ -365,20 +362,11 @@ __all__ = [  # noqa: F405
     "NamedAgg",
     "api",
     "read_xml",
+    "ArrowDtype",
+    "Flags",
+    "Float32Dtype",
+    "Float64Dtype",
+    "from_dummies",
 ]
 
-if PandasCompatVersion.CURRENT != PandasCompatVersion.PY36:
-    from modin._compat.pandas_api.namespace import (
-        ArrowDtype,
-        Flags,
-        Float32Dtype,
-        Float64Dtype,
-        from_dummies,
-    )
-
-    __all__.extend(
-        ["ArrowDtype", "Flags", "Float32Dtype", "Float64Dtype", "from_dummies"]
-    )
-del PandasCompatVersion
-
-del pandas, Engine, Parameter
+del pandas, Parameter
