@@ -182,140 +182,71 @@ def merge_asof(
     ):
         raise ValueError("Can't combine left/right_index with left/right_on or on.")
 
-    # Pandas fallbacks for tricky cases:
-    if (
-        # No idea how this works or why it does what it does; and in fact
-        # there's a Pandas bug suggesting it's wrong:
-        # https://github.com/pandas-dev/pandas/issues/33463
-        (left_index and right_on is not None)
-        # This is the case where by is a list of columns. If we're copying lots
-        # of columns out of Pandas, maybe not worth trying our path, it's not
-        # clear it's any better:
-        or not isinstance(by, (str, type(None)))
-        or not isinstance(left_by, (str, type(None)))
-        or not isinstance(right_by, (str, type(None)))
-        # The implementation below assumes that the right index is unique
-        # because it uses merge_asof to map each position in the merged
-        # index to the label of the one right row that should be merged
-        # at that row position.
-        or not right.index.is_unique
-    ):
-        if isinstance(right, DataFrame):
-            right = to_pandas(right)
-        return DataFrame(
-            pandas.merge_asof(
-                to_pandas(left),
-                right,
-                on=on,
-                left_on=left_on,
-                right_on=right_on,
-                left_index=left_index,
-                right_index=right_index,
-                by=by,
-                left_by=left_by,
-                right_by=right_by,
-                suffixes=suffixes,
-                tolerance=tolerance,
-                allow_exact_matches=allow_exact_matches,
-                direction=direction,
-            )
-        )
-
-    left_column = None
-    right_column = None
-
     if on is not None:
         if left_on is not None or right_on is not None:
             raise ValueError("If 'on' is set, 'left_on' and 'right_on' can't be set.")
         left_on = on
         right_on = on
 
-    if left_on is not None:
-        left_column = to_pandas(left[left_on])
-    elif left_index:
-        left_column = left.index
-    else:
-        raise ValueError("Need some sort of 'on' spec")
-
-    if right_on is not None:
-        right_column = to_pandas(right[right_on])
-    elif right_index:
-        right_column = right.index
-    else:
-        raise ValueError("Need some sort of 'on' spec")
-
-    # If we haven't set these by now, there's a bug in this function.
-    assert left_column is not None
-    assert right_column is not None
-
     if by is not None:
         if left_by is not None or right_by is not None:
             raise ValueError("Can't have both 'by' and 'left_by' or 'right_by'")
         left_by = right_by = by
 
-    # List of columns case should have been handled by direct Pandas fallback
-    # earlier:
-    assert isinstance(left_by, (str, type(None)))
-    assert isinstance(right_by, (str, type(None)))
+    if left_on is None and not left_index:
+        raise ValueError("Must pass on, left_on, or left_index=True")
 
-    left_pandas_limited = {"on": left_column}
-    right_pandas_limited = {"on": right_column, "right_labels": right.index}
-    extra_kwargs = {}  # extra arguments to Pandas merge_asof
+    if right_on is None and not right_index:
+        raise ValueError("Must pass on, right_on, or right_index=True")
 
-    if left_by is not None or right_by is not None:
-        extra_kwargs["by"] = "by"
-        left_pandas_limited["by"] = to_pandas(left[left_by])
-        right_pandas_limited["by"] = to_pandas(right[right_by])
-
-    # 1. Construct Pandas DataFrames with just the 'on' and optional 'by'
-    # columns, and the index as another column.
-    left_pandas_limited = pandas.DataFrame(left_pandas_limited, index=left.index)
-    right_pandas_limited = pandas.DataFrame(right_pandas_limited)
-
-    # 2. Use Pandas' merge_asof to figure out how to map labels on left to
-    # labels on the right.
-    merged = pandas.merge_asof(
-        left_pandas_limited,
-        right_pandas_limited,
-        on="on",
-        direction=direction,
-        allow_exact_matches=allow_exact_matches,
-        tolerance=tolerance,
-        **extra_kwargs,
-    )
-    # Now merged["right_labels"] shows which labels from right map to left's index.
-
-    # 3. Re-index right using the merged["right_labels"]; at this point right
-    # should be same length and (semantically) same order as left:
-    right_subset = right.reindex(index=pandas.Index(merged["right_labels"]))
-    if not right_index:
-        right_subset.drop(columns=[right_on], inplace=True)
-    if right_by is not None and left_by == right_by:
-        right_subset.drop(columns=[right_by], inplace=True)
-    right_subset.index = left.index
-
-    # 4. Merge left and the new shrunken right:
-    result = merge(
-        left,
-        right_subset,
-        left_index=True,
-        right_index=True,
-        suffixes=suffixes,
-        how="left",
-    )
-
-    # 5. Clean up to match Pandas output:
-    if left_on is not None and right_index:
-        result.insert(
-            # In theory this could use get_indexer_for(), but that causes an error:
-            list(result.columns).index(left_on + suffixes[0]),
+    return DataFrame(
+        query_compiler=left._query_compiler.merge_asof(
+            right._query_compiler,
             left_on,
-            result[left_on + suffixes[0]],
+            right_on,
+            left_index,
+            right_index,
+            left_by,
+            right_by,
+            suffixes,
+            tolerance,
+            allow_exact_matches,
+            direction,
         )
-    if not left_index and not right_index:
-        result.index = pandas.RangeIndex(start=0, stop=len(result))
+    )
 
-    return result
+
+@_inherit_docstrings(pandas.pivot_table, apilink="pandas.pivot_table")
+@enable_logging
+def pivot_table(
+    data,
+    values=None,
+    index=None,
+    columns=None,
+    aggfunc="mean",
+    fill_value=None,
+    margins=False,
+    dropna=True,
+    margins_name="All",
+    observed=False,
+    sort=True,
+):
+    if not isinstance(data, DataFrame):
+        raise ValueError(
+            "can not create pivot table with instance of type {}".format(type(data))
+        )
+
+    return data.pivot_table(
+        values=values,
+        index=index,
+        columns=columns,
+        aggfunc=aggfunc,
+        fill_value=fill_value,
+        margins=margins,
+        dropna=dropna,
+        margins_name=margins_name,
+        sort=sort,
+    )
 
 
 @_inherit_docstrings(pandas.pivot, apilink="pandas.pivot")
@@ -340,6 +271,25 @@ def to_numeric(arg, errors="raise", downcast=None):  # noqa: PR01, RT01, D200
     return arg._to_numeric(errors=errors, downcast=downcast)
 
 
+@_inherit_docstrings(pandas.qcut, apilink="pandas.qcut")
+@enable_logging
+def qcut(
+    x, q, labels=None, retbins=False, precision=3, duplicates="raise"
+):  # noqa: PR01, RT01, D200
+    """
+    Quantile-based discretization function.
+    """
+    kwargs = {
+        "labels": labels,
+        "retbins": retbins,
+        "precision": precision,
+        "duplicates": duplicates,
+    }
+    if not isinstance(x, Series):
+        return pandas.qcut(x, q, **kwargs)
+    return x._qcut(q, **kwargs)
+
+
 @_inherit_docstrings(pandas.unique, apilink="pandas.unique")
 @enable_logging
 def unique(values):  # noqa: PR01, RT01, D200
@@ -411,13 +361,13 @@ def concat(
         )
     axis = pandas.DataFrame()._get_axis_number(axis)
     if isinstance(objs, dict):
-        list_of_objs = list(objs.values())
+        input_list_of_objs = list(objs.values())
     else:
-        list_of_objs = list(objs)
-    if len(list_of_objs) == 0:
+        input_list_of_objs = list(objs)
+    if len(input_list_of_objs) == 0:
         raise ValueError("No objects to concatenate")
 
-    list_of_objs = [obj for obj in list_of_objs if obj is not None]
+    list_of_objs = [obj for obj in input_list_of_objs if obj is not None]
 
     if len(list_of_objs) == 0:
         raise ValueError("All objects passed were None")
@@ -454,26 +404,31 @@ def concat(
                 sort=sort,
             )
         )
-    if join not in ["inner", "outer"]:
+    if join == "outer":
+        # Filter out empties
+        list_of_objs = [
+            obj
+            for obj in list_of_objs
+            if (
+                isinstance(obj, (Series, pandas.Series))
+                or (isinstance(obj, DataFrame) and obj._query_compiler.lazy_execution)
+                or sum(obj.shape) > 0
+            )
+        ]
+    elif join != "inner":
         raise ValueError(
             "Only can inner (intersect) or outer (union) join the other axis"
         )
     # We have the weird Series and axis check because, when concatenating a
     # dataframe to a series on axis=0, pandas ignores the name of the series,
     # and this check aims to mirror that (possibly buggy) functionality
-    list_of_objs = [
-        obj
-        if isinstance(obj, DataFrame)
-        else DataFrame(obj.rename())
-        if isinstance(obj, (pandas.Series, Series)) and axis == 0
-        else DataFrame(obj)
-        for obj in list_of_objs
-    ]
     list_of_objs = [
         obj._query_compiler
+        if isinstance(obj, DataFrame)
+        else DataFrame(obj.rename())._query_compiler
+        if isinstance(obj, (pandas.Series, Series)) and axis == 0
+        else DataFrame(obj)._query_compiler
         for obj in list_of_objs
-        if (not obj._query_compiler.lazy_execution and len(obj.index))
-        or len(obj.columns)
     ]
     if keys is not None:
         if all_series:
@@ -504,6 +459,14 @@ def concat(
         ).index
     else:
         new_idx = None
+
+    if len(list_of_objs) == 0:
+        return DataFrame(
+            index=input_list_of_objs[0].index.append(
+                [f.index for f in input_list_of_objs[1:]]
+            )
+        )
+
     new_query_compiler = list_of_objs[0].concat(
         axis,
         list_of_objs[1:],
