@@ -13,8 +13,12 @@
 
 """Implement GroupBy public API as pandas does."""
 
+import warnings
+
 import numpy as np
 import pandas
+from pandas.core.apply import reconstruct_func
+from pandas.errors import SpecificationError
 import pandas.core.groupby
 from pandas.core.dtypes.common import is_list_like, is_numeric_dtype
 from pandas._libs.lib import no_default
@@ -23,6 +27,7 @@ from types import BuiltinFunctionType
 from collections.abc import Iterable
 
 from modin.error_message import ErrorMessage
+from modin.logging import ClassLogger
 from modin.utils import (
     _inherit_docstrings,
     try_cast_to_pandas,
@@ -36,13 +41,45 @@ from modin.core.dataframe.algebra.default2pandas.groupby import GroupBy
 from modin.config import IsExperimental
 from .series import Series
 from .utils import is_label
-from modin._compat import PandasCompatVersion
-from modin._compat.core.pandas_common import reconstruct_func
-from modin._compat.pandas_api.classes import DataFrameGroupByCompat, SeriesGroupByCompat
+
+
+_DEFAULT_BEHAVIOUR = {
+    "__class__",
+    "__getitem__",
+    "__init__",
+    "__iter__",
+    "_as_index",
+    "_axis",
+    "_by",
+    "_check_index",
+    "_check_index_name",
+    "_columns",
+    "_compute_index_grouped",
+    "_default_to_pandas",
+    "_df",
+    "_drop",
+    "_groups_cache",
+    "_idx_name",
+    "_index",
+    "_indices_cache",
+    "_internal_by",
+    "_internal_by_cache",
+    "_is_multi_by",
+    "_iter",
+    "_kwargs",
+    "_level",
+    "_pandas_class",
+    "_query_compiler",
+    "_sort",
+    "_squeeze",
+    "_wrap_aggregation",
+}
 
 
 @_inherit_docstrings(pandas.core.groupby.DataFrameGroupBy)
-class DataFrameGroupBy(DataFrameGroupByCompat):
+class DataFrameGroupBy(ClassLogger):
+    _pandas_class = pandas.core.groupby.DataFrameGroupBy
+
     def __init__(
         self,
         df,
@@ -115,16 +152,69 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
                 return self.__getitem__(key)
             raise err
 
+    # TODO: `.__getattribute__` overriding is broken in experimental mode. We should
+    # remove this branching one it's fixed:
+    # https://github.com/modin-project/modin/issues/5536
+    if not IsExperimental.get():
+
+        def __getattribute__(self, item):
+            attr = super().__getattribute__(item)
+            if (
+                item not in _DEFAULT_BEHAVIOUR
+                and not self._query_compiler.lazy_execution
+            ):
+                # We default to pandas on empty DataFrames. This avoids a large amount of
+                # pain in underlying implementation and returns a result immediately rather
+                # than dealing with the edge cases that empty DataFrames have.
+                if (
+                    callable(attr)
+                    and self._df.empty
+                    and hasattr(self._pandas_class, item)
+                ):
+
+                    def default_handler(*args, **kwargs):
+                        return self._default_to_pandas(item, *args, **kwargs)
+
+                    return default_handler
+            return attr
+
     @property
     def ngroups(self):
         return len(self)
 
     def skew(self, *args, **kwargs):
-        return self._wrap_aggregation(
-            type(self._query_compiler).groupby_skew,
+        # The 'skew' aggregation is less tolerant to non-numeric columns than others
+        # (i.e. it doesn't allow numeric categoricals), thus dropping non-numeric
+        # columns here since `._wrap_aggregation(numeric_only=True, ...)` is not enough
+        if self.ndim == 2:
+            by_cols = self._internal_by
+            mask_cols = [
+                col
+                for col, dtype in self._df.dtypes.items()
+                if is_numeric_dtype(dtype) or col in by_cols
+            ]
+            if not self._df.columns.equals(mask_cols):
+                masked_df = self._df[mask_cols]
+                masked_obj = type(self)(
+                    df=masked_df,
+                    by=self._by,
+                    axis=self._axis,
+                    idx_name=self._idx_name,
+                    drop=self._drop,
+                    squeeze=self._squeeze,
+                    **self._kwargs,
+                )
+            else:
+                masked_obj = self
+        else:
+            masked_obj = self
+
+        return masked_obj._wrap_aggregation(
+            type(masked_obj._query_compiler).groupby_skew,
             agg_args=args,
             agg_kwargs=kwargs,
-            numeric_only=True,
+            # Don't want to try to drop non-numeric columns for the second time
+            numeric_only=False,
         )
 
     def ffill(self, limit=None):
@@ -133,6 +223,24 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
     def sem(self, ddof=1):
         return self._default_to_pandas(lambda df: df.sem(ddof=ddof))
 
+    def value_counts(
+        self,
+        subset=None,
+        normalize: bool = False,
+        sort: bool = True,
+        ascending: bool = False,
+        dropna: bool = True,
+    ):
+        return self._default_to_pandas(
+            lambda df: df.value_counts(
+                subset=subset,
+                normalize=normalize,
+                sort=sort,
+                ascending=ascending,
+                dropna=dropna,
+            )
+        )
+
     def mean(self, numeric_only=None):
         return self._check_index(
             self._wrap_aggregation(
@@ -236,12 +344,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
                 and DataFrame(query_compiler=self._by.isna()).any(axis=None)
             ):
                 mask_nan_rows = data[self._by.columns].isna().any(axis=1)
-                if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36:
-                    # older pandas filled invalid rows with NaN-s
-                    result.loc[mask_nan_rows] = np.nan
-                else:
-                    # drop NaN groups
-                    result = result.loc[~mask_nan_rows]
+                result = result.loc[~mask_nan_rows]
             return result
 
         if freq is None and axis == 1 and self._axis == 0:
@@ -255,16 +358,6 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
             result = _shift(
                 self._df, periods, freq, axis, fill_value, is_set_nan_rows=False
             )
-            if PandasCompatVersion.CURRENT != PandasCompatVersion.LATEST:
-                # pandas>=1.5 no longer make multi-index in groupby().shift()
-                new_idx_lvl_arrays = np.concatenate(
-                    [self._df[self._by.columns].values.T, [list(result.index)]]
-                )
-                result.index = pandas.MultiIndex.from_arrays(
-                    new_idx_lvl_arrays,
-                    names=[col_name for col_name in self._by.columns]
-                    + [result._query_compiler.get_index_name()],
-                )
             result = result.dropna(subset=self._by.columns)
             if self._sort:
                 result = result.sort_values(list(self._by.columns), axis=axis)
@@ -308,7 +401,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         return self._indices_cache
 
     @_inherit_docstrings(pandas.core.groupby.DataFrameGroupBy.pct_change)
-    def _pct_change(self, *args, **kwargs):
+    def pct_change(self, *args, **kwargs):
         return self._default_to_pandas(lambda df: df.pct_change(*args, **kwargs))
 
     def filter(self, func, dropna=True, *args, **kwargs):
@@ -355,6 +448,14 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         return self._default_to_pandas(lambda df: df.first(**kwargs))
 
     def backfill(self, limit=None):
+        warnings.warn(
+            (
+                "backfill is deprecated and will be removed in a future version. "
+                + "Use bfill instead."
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
         return self.bfill(limit)
 
     _internal_by_cache = no_default
@@ -507,7 +608,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         ):
             func = func.__name__
 
-        relabeling_required = False
+        do_relabel = None
         if isinstance(func, dict) or func is None:
 
             def try_get_str_func(fn):
@@ -518,6 +619,30 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
             relabeling_required, func_dict, new_columns, order = reconstruct_func(
                 func, **kwargs
             )
+
+            if relabeling_required:
+
+                def do_relabel(obj_to_relabel):
+                    new_order, new_columns_idx = order, pandas.Index(new_columns)
+                    if not self._as_index:
+                        nby_cols = len(obj_to_relabel.columns) - len(new_columns_idx)
+                        new_order = np.concatenate(
+                            [np.arange(nby_cols), new_order + nby_cols]
+                        )
+                        by_cols = obj_to_relabel.columns[:nby_cols]
+                        if by_cols.nlevels != new_columns_idx.nlevels:
+                            by_cols = by_cols.remove_unused_levels()
+                            empty_levels = [
+                                i
+                                for i, level in enumerate(by_cols.levels)
+                                if len(level) == 1 and level[0] == ""
+                            ]
+                            by_cols = by_cols.droplevel(empty_levels)
+                        new_columns_idx = by_cols.append(new_columns_idx)
+                    result = obj_to_relabel.iloc[:, new_order]
+                    result.columns = new_columns_idx
+                    return result
+
             func_dict = {col: try_get_str_func(fn) for col, fn in func_dict.items()}
             if any(isinstance(fn, list) for fn in func_dict.values()):
                 # multicolumn case
@@ -544,8 +669,6 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
                 )
 
             if any(i not in self._df.columns for i in func_dict.keys()):
-                from modin._compat.core.pandas_common import SpecificationError
-
                 raise SpecificationError("nested renamer is not supported")
             if func is None:
                 kwargs = {}
@@ -582,25 +705,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
             agg_kwargs=kwargs,
             how="axis_wise",
         )
-
-        if relabeling_required:
-            if not self._as_index:
-                nby_cols = len(result.columns) - len(new_columns)
-                order = np.concatenate([np.arange(nby_cols), order + nby_cols])
-                by_cols = result.columns[:nby_cols]
-                new_columns = pandas.Index(new_columns)
-                if by_cols.nlevels != new_columns.nlevels:
-                    by_cols = by_cols.remove_unused_levels()
-                    empty_levels = [
-                        i
-                        for i, level in enumerate(by_cols.levels)
-                        if len(level) == 1 and level[0] == ""
-                    ]
-                    by_cols = by_cols.droplevel(empty_levels)
-                new_columns = by_cols.append(new_columns)
-            result = result.iloc[:, order]
-            result.columns = new_columns
-        return result
+        return do_relabel(result) if do_relabel else result
 
     agg = aggregate
 
@@ -608,6 +713,14 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         return self._default_to_pandas(lambda df: df.last(**kwargs))
 
     def mad(self, **kwargs):
+        warnings.warn(
+            (
+                "The 'mad' method is deprecated and will be removed in a future version. "
+                + "To compute the same result, you may do `(df - df.mean()).abs().mean()`."
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
         return self._default_to_pandas(lambda df: df.mad(**kwargs))
 
     def rank(self, **kwargs):
@@ -681,7 +794,6 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         if not isinstance(result, Series):
             result = result.squeeze(axis=1)
         if not self._kwargs.get("as_index") and not isinstance(result, Series):
-            result = result.rename(columns={0: "size"})
             result = (
                 result.rename(columns={MODIN_UNNAMED_SERIES_LABEL: "index"})
                 if MODIN_UNNAMED_SERIES_LABEL in result.columns
@@ -859,8 +971,8 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
     def diff(self):
         return self._default_to_pandas(lambda df: df.diff())
 
-    def take(self, **kwargs):
-        return self._default_to_pandas(lambda df: df.take(**kwargs))
+    def take(self, *args, **kwargs):
+        return self._default_to_pandas(lambda df: df.take(*args, **kwargs))
 
     @property
     def _index(self):
@@ -981,7 +1093,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         # `dropna` param is the only one that matters for the group indices result
         dropna = self._kwargs.get("dropna", True)
 
-        if hasattr(self._by, "columns") and is_multi_by:
+        if isinstance(self._by, BaseQueryCompiler) and is_multi_by:
             by = list(self._by.columns)
 
         if is_multi_by:
@@ -1151,7 +1263,7 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
 
         Parameters
         ----------
-        f : callable
+        f : callable or str
             The function to apply to each group.
         *args : list
             Extra positional arguments to pass to `f`.
@@ -1181,19 +1293,28 @@ class DataFrameGroupBy(DataFrameGroupByCompat):
         by = GroupBy.validate_by(by)
 
         def groupby_on_multiple_columns(df, *args, **kwargs):
-            return f(
-                df.groupby(
-                    by=by, axis=self._axis, squeeze=self._squeeze, **self._kwargs
-                ),
-                *args,
-                **kwargs,
+            groupby_obj = df.groupby(
+                by=by, axis=self._axis, squeeze=self._squeeze, **self._kwargs
             )
 
+            if callable(f):
+                return f(groupby_obj, *args, **kwargs)
+            else:
+                ErrorMessage.catch_bugs_and_request_email(
+                    failure_condition=not isinstance(f, str)
+                )
+                attribute = getattr(groupby_obj, f)
+                if callable(attribute):
+                    return attribute(*args, **kwargs)
+                return attribute
+
         return self._df._default_to_pandas(groupby_on_multiple_columns, *args, **kwargs)
 
 
 @_inherit_docstrings(pandas.core.groupby.SeriesGroupBy)
-class SeriesGroupBy(SeriesGroupByCompat, DataFrameGroupBy):
+class SeriesGroupBy(DataFrameGroupBy):
+    _pandas_class = pandas.core.groupby.SeriesGroupBy
+
     @property
     def ndim(self):
         """
@@ -1247,6 +1368,24 @@ class SeriesGroupBy(SeriesGroupByCompat, DataFrameGroupBy):
                 for k in (sorted(group_ids) if self._sort else group_ids)
             )
 
+    def value_counts(
+        self,
+        normalize: bool = False,
+        sort: bool = True,
+        ascending: bool = False,
+        bins=None,
+        dropna: bool = True,
+    ):
+        return self._default_to_pandas(
+            lambda ser: ser.value_counts(
+                normalize=normalize,
+                sort=sort,
+                ascending=ascending,
+                bins=bins,
+                dropna=dropna,
+            )
+        )
+
 
 if IsExperimental.get():
     from modin.experimental.cloud.meta_magic import make_wrapped_class
