@@ -233,11 +233,6 @@ Location based indexing can only have [integer, integer slice (START point is
 INCLUDED, END point is EXCLUDED), listlike of integers, boolean array] types.
 """
 
-_VIEW_IS_COPY_WARNING = """
-Modin is making a copy of of the DataFrame. This behavior diverges from Pandas.
-This will be fixed in future releases.
-"""
-
 
 def _compute_ndim(row_loc, col_loc):
     """
@@ -319,10 +314,9 @@ class _LocationIndexerBase(ClassLogger):
         """
         raise NotImplementedError("Implemented by subclasses")
 
-    def _getitem_positional(
+    def _get_pandas_object_from_qc_view(
         self,
-        row_lookup,
-        col_lookup,
+        qc_view,
         row_multiindex_full_lookup: bool,
         col_multiindex_full_lookup: bool,
         row_scalar: bool,
@@ -330,29 +324,27 @@ class _LocationIndexerBase(ClassLogger):
         ndim: int,
     ):
         """
-        Retrieve dataset according to `row_lookup` and `col_lookup`.
+        Convert the query compiler view to the appropriate pandas object.
 
         Parameters
         ----------
-        row_lookup : slice(None), range or np.ndarray
-            The global row index to retrieve data from.
-        col_lookup : slice(None), range or np.ndarray
-            The global col index to retrieve data from.
+        qc_view : BaseQueryCompiler
+            Query compiler to convert.
         row_multiindex_full_lookup : bool
             See _multiindex_possibly_contains_key.__doc__.
         col_multiindex_full_lookup : bool
             See _multiindex_possibly_contains_key.__doc__.
         row_scalar : bool
-            Whether indexer for rows is scalar or not.
+            Whether indexer for rows is scalar.
         col_scalar : bool
-            Whether indexer for columns is scalar or not.
+            Whether indexer for columns is scalar.
         ndim : {0, 1, 2}
             Number of dimensions in dataset to be retrieved.
 
         Returns
         -------
         modin.pandas.DataFrame or modin.pandas.Series
-            Located dataset.
+            The pandas object with the data from the query compiler view.
 
         Notes
         -----
@@ -361,21 +353,6 @@ class _LocationIndexerBase(ClassLogger):
         Ideally, this API should get rid of using slices as indexers and either use a
         common ``Indexer`` object or range and ``np.ndarray`` only.
         """
-        if isinstance(row_lookup, slice):
-            ErrorMessage.catch_bugs_and_request_email(
-                failure_condition=row_lookup != slice(None),
-                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {row_lookup}",
-            )
-            row_lookup = None
-        if isinstance(col_lookup, slice):
-            ErrorMessage.catch_bugs_and_request_email(
-                failure_condition=col_lookup != slice(None),
-                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {col_lookup}",
-            )
-            col_lookup = None
-
-        qc_view = self.qc.take_2d(row_lookup, col_lookup)
-
         if ndim == 2:
             return self.df.__constructor__(query_compiler=qc_view)
         if isinstance(self.df, Series) and not row_scalar:
@@ -655,7 +632,47 @@ class _LocIndexer(_LocationIndexerBase):
         """
         if self.df.empty:
             return self.df._default_to_pandas(lambda df: df.loc[key])
-        row_loc, col_loc, ndim = self._parse_row_and_column_locators(key)
+
+        if (
+            isinstance(key, tuple)
+            and len(key) == 2
+            and all((is_scalar(k) for k in key))
+            and self.qc.has_multiindex(axis=0)
+        ):
+            # __getitem__ has no way to distinguish between
+            # loc[('level_one_key', level_two_key')] and
+            # loc['level_one_key', 'column_name']. It's possible for both to be valid
+            # when we have a multiindex on axis=0, and it seems pandas uses
+            # interpretation 1 if that's possible. Do the same.
+            locators = self._parse_row_and_column_locators((key, slice(None)))
+            try:
+                return self._helper_for__getitem__(key, *locators)
+            except KeyError:
+                pass
+        return self._helper_for__getitem__(
+            key, *self._parse_row_and_column_locators(key)
+        )
+
+    def _helper_for__getitem__(self, key, row_loc, col_loc, ndim):
+        """
+        Retrieve dataset according to `key`, row_loc, and col_loc.
+
+        Parameters
+        ----------
+        key : callable, scalar, or tuple
+            The global row index to retrieve data from.
+        row_loc : callable, scalar, or slice
+            Row locator(s) as a scalar or List.
+        col_loc : callable, scalar, or slice
+            Row locator(s) as a scalar or List.
+        ndim : int
+            The number of dimensions of the returned object.
+
+        Returns
+        -------
+        modin.pandas.DataFrame or modin.pandas.Series
+            Located dataset.
+        """
         row_scalar = is_scalar(row_loc)
         col_scalar = is_scalar(col_loc)
 
@@ -678,11 +695,9 @@ class _LocIndexer(_LocationIndexerBase):
         if isinstance(row_loc, Series) and is_boolean_array(row_loc):
             return self._handle_boolean_masking(row_loc, col_loc)
 
-        row_lookup, col_lookup = self._compute_lookup(row_loc, col_loc)
-
-        result = self._getitem_positional(
-            row_lookup,
-            col_lookup,
+        qc_view = self.qc.take_2d_labels(row_loc, col_loc)
+        result = self._get_pandas_object_from_qc_view(
+            qc_view,
             row_multiindex_full_lookup,
             col_multiindex_full_lookup,
             row_scalar,
@@ -718,7 +733,7 @@ class _LocIndexer(_LocationIndexerBase):
             ):
                 result.index = result.index.droplevel(list(range(len(row_loc_as_list))))
         if (
-            hasattr(result, "columns")
+            isinstance(result, DataFrame)
             and not isinstance(col_loc_as_list, slice)
             and not levels_already_dropped
             and result._query_compiler.has_multiindex(axis=1)
@@ -731,7 +746,7 @@ class _LocIndexer(_LocationIndexerBase):
         # This is done for cases where the index passed in has other state, like a
         # frequency in the case of DateTimeIndex.
         if (
-            row_lookup is not None
+            row_loc is not None
             and isinstance(col_loc, slice)
             and col_loc == slice(None)
             and isinstance(key, pandas.Index)
@@ -764,32 +779,141 @@ class _LocIndexer(_LocationIndexerBase):
                 new_query_compiler=self.df._default_to_pandas(_loc)._query_compiler
             )
             return
-        row_loc, col_loc, _ = self._parse_row_and_column_locators(key)
-        if isinstance(row_loc, list) and len(row_loc) == 1:
-            if row_loc[0] not in self.qc.index:
-                index = self.qc.index.insert(len(self.qc.index), row_loc[0])
-                self.qc = self.qc.reindex(labels=index, axis=0)
-                self.df._update_inplace(new_query_compiler=self.qc)
+        row_loc, col_loc, ndims = self._parse_row_and_column_locators(key)
+        append_axis = self._check_missing_loc(row_loc, col_loc)
+        if ndims >= 1 and append_axis is not None:
+            # We enter this codepath if we're either appending a row or a column
+            if append_axis:
+                # Appending at least one new column
+                if is_scalar(col_loc):
+                    col_loc = [col_loc]
+                self._setitem_with_new_columns(row_loc, col_loc, item)
+            else:
+                # Appending at most one new row
+                if is_scalar(row_loc) or len(row_loc) == 1:
+                    index = self.qc.index.insert(len(self.qc.index), row_loc)
+                    self.qc = self.qc.reindex(labels=index, axis=0, fill_value=0)
+                    self.df._update_inplace(new_query_compiler=self.qc)
+                self._set_item_existing_loc(row_loc, col_loc, item)
+        else:
+            self._set_item_existing_loc(row_loc, col_loc, item)
 
+    def _setitem_with_new_columns(self, row_loc, col_loc, item):
+        """
+        Assign `item` value to dataset located by `row_loc` and `col_loc` with new columns.
+
+        Parameters
+        ----------
+        row_loc : scalar, slice, list, array or tuple
+            Row locator.
+        col_loc : scalar, slice, list, array or tuple
+            Columns locator.
+        item : modin.pandas.DataFrame, modin.pandas.Series or scalar
+            Value that should be assigned to located dataset.
+        """
+        exist_items = item
+        common_label_loc = np.isin(col_loc, self.qc.columns.values)
+        if is_list_like(item) and not isinstance(item, (DataFrame, Series)):
+            item = np.array(item)
+            if len(item.shape) == 1:
+                if item.shape[0] != len(col_loc):
+                    raise ValueError(
+                        "Must have equal len keys and value when setting with an iterable"
+                    )
+            else:
+                if item.shape != (len(self.qc.index, len(col_loc))):
+                    raise ValueError(
+                        "Must have equal len keys and value when setting with an iterable"
+                    )
+            exist_items = (
+                item[:, common_label_loc]
+                if len(item.shape) > 1
+                else item[common_label_loc]
+            )
+        if not all(common_label_loc):
+            # In this case we have some new cols and some old ones
+            columns = self.qc.columns
+            for i in range(len(common_label_loc)):
+                if not common_label_loc[i]:
+                    columns = columns.insert(len(columns), col_loc[i])
+            self.qc = self.qc.reindex(labels=columns, axis=1, fill_value=0)
+            self.df._update_inplace(new_query_compiler=self.qc)
+        self._set_item_existing_loc(row_loc, np.array(col_loc), exist_items)
+
+    def _set_item_existing_loc(self, row_loc, col_loc, item):
+        """
+        Assign `item` value to dataset located by `row_loc` and `col_loc` with existing rows and columns.
+
+        Parameters
+        ----------
+        row_loc : scalar, slice, list, array or tuple
+            Row locator.
+        col_loc : scalar, slice, list, array or tuple
+            Columns locator.
+        item : modin.pandas.DataFrame, modin.pandas.Series or scalar
+            Value that should be assigned to located dataset.
+        """
+        if (
+            isinstance(row_loc, Series)
+            and is_boolean_array(row_loc)
+            and is_scalar(item)
+        ):
+            new_qc = self.df._query_compiler.setitem_bool(
+                row_loc._query_compiler, col_loc, item
+            )
+            self.df._update_inplace(new_qc)
+            return
+
+        row_lookup, col_lookup = self.qc.get_positions_from_labels(row_loc, col_loc)
+        self._setitem_positional(
+            row_lookup,
+            col_lookup,
+            item,
+            axis=self._determine_setitem_axis(
+                row_lookup, col_lookup, is_scalar(row_loc), is_scalar(col_loc)
+            ),
+        )
+
+    def _check_missing_loc(self, row_loc, col_loc):
+        """
+        Help `__setitem__` compute whether an axis needs appending.
+
+        Parameters
+        ----------
+        row_loc : scalar, slice, list, array or tuple
+            Row locator.
+        col_loc : scalar, slice, list, array or tuple
+            Columns locator.
+
+        Returns
+        -------
+        int or None :
+            0 if new row, 1 if new column, None if neither.
+        """
+        if is_scalar(row_loc):
+            return 0 if row_loc not in self.qc.index else None
+        elif isinstance(row_loc, list):
+            missing_labels = self._compute_enlarge_labels(
+                pandas.Index(row_loc), self.qc.index
+            )
+            if len(missing_labels) > 1:
+                # We cast to list to copy pandas' error:
+                # In pandas, we get: KeyError: [a, b,...] not in index
+                # If we don't convert to list we get: KeyError: [a b ...] not in index
+                raise KeyError("{} not in index".format(list(missing_labels)))
+        if (
+            not (is_list_like(row_loc) or isinstance(row_loc, slice))
+            and row_loc not in self.qc.index
+        ):
+            return 0
         if (
             isinstance(col_loc, list)
-            and len(col_loc) == 1
-            and col_loc[0] not in self.qc.columns
+            and len(pandas.Index(col_loc).difference(self.qc.columns)) >= 1
         ):
-            new_col = pandas.Series(index=self.df.index)
-            new_col[row_loc] = item
-            self.df.insert(loc=len(self.df.columns), column=col_loc[0], value=new_col)
-            self.qc = self.df._query_compiler
-        else:
-            row_lookup, col_lookup = self._compute_lookup(row_loc, col_loc)
-            self._setitem_positional(
-                row_lookup,
-                col_lookup,
-                item,
-                axis=self._determine_setitem_axis(
-                    row_lookup, col_lookup, is_scalar(row_loc), is_scalar(col_loc)
-                ),
-            )
+            return 1
+        if is_scalar(col_loc) and col_loc not in self.qc.columns:
+            return 1
+        return None
 
     def _compute_enlarge_labels(self, locator, base_index):
         """
@@ -810,112 +934,28 @@ class _LocIndexer(_LocationIndexerBase):
         # base_index_type can be pd.Index or pd.DatetimeIndex
         # depending on user input and pandas behavior
         # See issue #2264
-        base_index_type = type(base_index)
-        locator_as_index = base_index_type(locator)
+        base_as_index = pandas.Index(list(base_index))
+        locator_as_index = pandas.Index(list(locator))
 
-        nan_labels = locator_as_index.difference(base_index)
-        common_labels = locator_as_index.intersection(base_index)
+        if locator_as_index.inferred_type == "boolean":
+            if len(locator_as_index) != len(base_as_index):
+                raise ValueError(
+                    f"Item wrong length {len(locator_as_index)} instead of {len(base_as_index)}!"
+                )
+            common_labels = base_as_index[locator_as_index]
+            nan_labels = pandas.Index([])
+        else:
+            common_labels = locator_as_index.intersection(base_as_index)
+            nan_labels = locator_as_index.difference(base_as_index)
 
         if len(common_labels) == 0:
             raise KeyError(
                 "None of [{labels}] are in the [{base_index_name}]".format(
-                    labels=list(locator_as_index), base_index_name=base_index
+                    labels=list(locator_as_index), base_index_name=base_as_index
                 )
             )
         return nan_labels
 
-    def _compute_lookup(self, row_loc, col_loc):
-        """
-        Compute index and column labels from index and column locators.
-
-        Parameters
-        ----------
-        row_loc : scalar, slice, list, array or tuple
-            Row locator.
-        col_loc : scalar, slice, list, array or tuple
-            Columns locator.
-
-        Returns
-        -------
-        row_lookup : slice(None) if full axis grab, pandas.RangeIndex if repetition is detected, numpy.ndarray otherwise
-            List of index labels.
-        col_lookup : slice(None) if full axis grab, pandas.RangeIndex if repetition is detected, numpy.ndarray otherwise
-            List of columns labels.
-
-        Notes
-        -----
-        Usage of `slice(None)` as a resulting lookup is a hack to pass information about
-        full-axis grab without computing actual indices that triggers lazy computations.
-        Ideally, this API should get rid of using slices as indexers and either use a
-        common ``Indexer`` object or range and ``np.ndarray`` only.
-        """
-        lookups = []
-        for axis, axis_loc in enumerate((row_loc, col_loc)):
-            if is_scalar(axis_loc):
-                axis_loc = np.array([axis_loc])
-            if isinstance(axis_loc, slice) or is_range_like(axis_loc):
-                if isinstance(axis_loc, slice) and axis_loc == slice(None):
-                    axis_lookup = axis_loc
-                else:
-                    axis_labels = self.qc.get_axis(axis)
-                    # `slice_indexer` returns a fully-defined numeric slice for a non-fully-defined labels-based slice
-                    axis_lookup = axis_labels.slice_indexer(
-                        axis_loc.start, axis_loc.stop, axis_loc.step
-                    )
-                    # Converting negative indices to their actual positions:
-                    axis_lookup = pandas.RangeIndex(
-                        start=(
-                            axis_lookup.start
-                            if axis_lookup.start >= 0
-                            else axis_lookup.start + len(axis_labels)
-                        ),
-                        stop=(
-                            axis_lookup.stop
-                            if axis_lookup.stop >= 0
-                            else axis_lookup.stop + len(axis_labels)
-                        ),
-                        step=axis_lookup.step,
-                    )
-            elif self.qc.has_multiindex(axis):
-                # `Index.get_locs` raises an IndexError by itself if missing labels were provided,
-                # we don't have to do missing-check for the received `axis_lookup`.
-                if isinstance(axis_loc, pandas.MultiIndex):
-                    axis_lookup = self.qc.get_axis(axis).get_indexer_for(axis_loc)
-                else:
-                    axis_lookup = self.qc.get_axis(axis).get_locs(axis_loc)
-            elif is_boolean_array(axis_loc):
-                axis_lookup = boolean_mask_to_numeric(axis_loc)
-            else:
-                axis_labels = self.qc.get_axis(axis)
-                if is_list_like(axis_loc) and not isinstance(
-                    axis_loc, (np.ndarray, pandas.Index)
-                ):
-                    # `Index.get_indexer_for` works much faster with numpy arrays than with python lists,
-                    # so although we lose some time here on converting to numpy, `Index.get_indexer_for`
-                    # speedup covers the loss that we gain here.
-                    axis_loc = np.array(axis_loc, dtype=axis_labels.dtype)
-                axis_lookup = axis_labels.get_indexer_for(axis_loc)
-                # `Index.get_indexer_for` sets -1 value for missing labels, we have to verify whether
-                # there are any -1 in the received indexer to raise a KeyError here.
-                missing_mask = axis_lookup < 0
-                if missing_mask.any():
-                    missing_labels = (
-                        # Converting `axis_loc` to maskable `np.array` to not fail
-                        # on masking non-maskable list-like
-                        np.array(axis_loc)[missing_mask]
-                        if is_list_like(axis_loc)
-                        # If `axis_loc` is not a list-like then we can't select certain
-                        # labels that are missing and so printing the whole indexer
-                        else axis_loc
-                    )
-                    raise KeyError(missing_labels)
-
-            if isinstance(axis_lookup, pandas.Index) and not is_range_like(axis_lookup):
-                axis_lookup = axis_lookup.values
-
-            lookups.append(axis_lookup)
-        return lookups
-
 
 class _iLocIndexer(_LocationIndexerBase):
     """
@@ -957,10 +997,21 @@ class _iLocIndexer(_LocationIndexerBase):
             return self._handle_boolean_masking(row_loc, col_loc)
 
         row_lookup, col_lookup = self._compute_lookup(row_loc, col_loc)
-
-        result = self._getitem_positional(
-            row_lookup,
-            col_lookup,
+        if isinstance(row_lookup, slice):
+            ErrorMessage.catch_bugs_and_request_email(
+                failure_condition=row_lookup != slice(None),
+                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {row_lookup}",
+            )
+            row_lookup = None
+        if isinstance(col_lookup, slice):
+            ErrorMessage.catch_bugs_and_request_email(
+                failure_condition=col_lookup != slice(None),
+                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {col_lookup}",
+            )
+            col_lookup = None
+        qc_view = self.qc.take_2d_positional(row_lookup, col_lookup)
+        result = self._get_pandas_object_from_qc_view(
+            qc_view,
             row_multiindex_full_lookup=False,
             col_multiindex_full_lookup=False,
             row_scalar=row_scalar,
