@@ -13,6 +13,7 @@
 
 """Module houses `Series` class, that is distributed version of `pandas.Series`."""
 
+from __future__ import annotations
 import numpy as np
 import pandas
 from pandas.api.types import is_integer
@@ -22,9 +23,10 @@ from pandas.core.dtypes.common import (
     is_dict_like,
     is_list_like,
 )
-from pandas._libs.lib import no_default
-from pandas._typing import IndexKeyFunc
-from typing import Union, Optional, Hashable, TYPE_CHECKING
+from pandas.core.series import _coerce_method
+from pandas._libs.lib import no_default, NoDefault
+from pandas._typing import IndexKeyFunc, Axis
+from typing import Union, Optional, Hashable, TYPE_CHECKING, IO
 import warnings
 
 from modin.utils import (
@@ -39,7 +41,6 @@ from .iterator import PartitionIterator
 from .utils import from_pandas, is_scalar, _doc_binary_op
 from .accessor import CachedAccessor, SparseAccessor
 from . import _update_engine
-from modin._compat.pandas_api.classes import SeriesCompat
 
 
 if TYPE_CHECKING:
@@ -49,7 +50,7 @@ if TYPE_CHECKING:
 @_inherit_docstrings(
     pandas.Series, excluded=[pandas.Series.__init__], apilink="pandas.Series"
 )
-class Series(SeriesCompat, BasePandasDataset):
+class Series(BasePandasDataset):
     """
     Modin distributed representation of `pandas.Series`.
 
@@ -92,6 +93,8 @@ class Series(SeriesCompat, BasePandasDataset):
         fastpath=False,
         query_compiler=None,
     ):
+        from modin.numpy import array
+
         # Siblings are other dataframes that share the same query compiler. We
         # use this list to update inplace when there is a shallow copy.
         self._siblings = []
@@ -105,6 +108,18 @@ class Series(SeriesCompat, BasePandasDataset):
                         + "not yet implemented."
                     )
                 query_compiler = data.loc[index]._query_compiler
+        if isinstance(data, array):
+            if data._ndim == 2:
+                raise ValueError("Data must be 1-dimensional")
+            query_compiler = data._query_compiler.copy()
+            if index is not None:
+                query_compiler.index = index
+            if dtype is not None:
+                query_compiler = query_compiler.astype(
+                    {col_name: dtype for col_name in query_compiler.columns}
+                )
+            if name is None:
+                query_compiler.columns = pandas.Index([MODIN_UNNAMED_SERIES_LABEL])
         if query_compiler is None:
             # Defaulting to pandas
             warnings.warn(
@@ -129,7 +144,6 @@ class Series(SeriesCompat, BasePandasDataset):
             )._query_compiler
         self._query_compiler = query_compiler.columnarize()
         if name is not None:
-            self._query_compiler = self._query_compiler
             self.name = name
 
     def _get_name(self):
@@ -272,16 +286,6 @@ class Series(SeriesCompat, BasePandasDataset):
     def __rdivmod__(self, left):
         return self.rdivmod(left)
 
-    def __float__(self):
-        """
-        Return float representation of Series.
-
-        Returns
-        -------
-        float
-        """
-        return float(self.squeeze())
-
     @_doc_binary_op(operation="integer division", bin_op="floordiv")
     def __floordiv__(self, right):
         return self.floordiv(right)
@@ -315,15 +319,8 @@ class Series(SeriesCompat, BasePandasDataset):
                 return self[key]
             raise err
 
-    def __int__(self):
-        """
-        Return integer representation of Series.
-
-        Returns
-        -------
-        int
-        """
-        return int(self.squeeze())
+    __float__ = _coerce_method(float)
+    __int__ = _coerce_method(int)
 
     def __iter__(self):
         """
@@ -421,7 +418,14 @@ class Series(SeriesCompat, BasePandasDataset):
         )
         if len(self) == 0:
             return "Series([], {}{}{}".format(freq_str, name_str, dtype_str)
-        return temp_str.rsplit("\n", 1)[0] + "\n{}{}{}{}".format(
+        maxsplit = 1
+        if (
+            isinstance(temp_df, pandas.Series)
+            and temp_df.name is not None
+            and temp_df.dtype == "category"
+        ):
+            maxsplit = 2
+        return temp_str.rsplit("\n", maxsplit)[0] + "\n{}{}{}{}".format(
             freq_str, name_str, len_str, dtype_str
         )
 
@@ -485,7 +489,21 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Return Series as ndarray or ndarray-like depending on the dtype.
         """
-        return self.to_numpy()
+        import modin.pandas as pd
+
+        if isinstance(
+            self.dtype, pandas.core.dtypes.dtypes.ExtensionDtype
+        ) and not isinstance(self.dtype, pd.CategoricalDtype):
+            return self._default_to_pandas("values")
+
+        data = self.to_numpy()
+        if isinstance(self.dtype, pd.CategoricalDtype):
+            from modin.config import ExperimentalNumPyAPI
+
+            if ExperimentalNumPyAPI.get():
+                data = data._to_numpy()
+            data = pd.Categorical(data, dtype=self.dtype)
+        return data
 
     def add(self, other, level=None, fill_value=None, axis=0):  # noqa: PR01, RT01, D200
         """
@@ -511,13 +529,17 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Prefix labels with string `prefix`.
         """
-        return Series(query_compiler=self._query_compiler.add_prefix(prefix, axis=0))
+        return self.__constructor__(
+            query_compiler=self._query_compiler.add_prefix(prefix, axis=0)
+        )
 
     def add_suffix(self, suffix):  # noqa: PR01, RT01, D200
         """
         Suffix labels with string `suffix`.
         """
-        return Series(query_compiler=self._query_compiler.add_suffix(suffix, axis=0))
+        return self.__constructor__(
+            query_compiler=self._query_compiler.add_suffix(suffix, axis=0)
+        )
 
     def append(
         self, to_append, ignore_index=False, verify_integrity=False
@@ -593,7 +615,7 @@ class Series(SeriesCompat, BasePandasDataset):
         if len(query_compiler.columns) > 1:
             return DataFrame(query_compiler=query_compiler)
         else:
-            return Series(query_compiler=query_compiler)
+            return self.__constructor__(query_compiler=query_compiler)
 
     def aggregate(self, func=None, axis=0, *args, **kwargs):  # noqa: PR01, RT01, D200
         """
@@ -612,7 +634,9 @@ class Series(SeriesCompat, BasePandasDataset):
 
     agg = aggregate
 
-    def _apply(self, func, convert_dtype, args, **kwargs):  # noqa: PR01, RT01, D200
+    def apply(
+        self, func, convert_dtype=True, args=(), **kwargs
+    ):  # noqa: PR01, RT01, D200
         """
         Invoke function on values of Series.
         """
@@ -648,8 +672,7 @@ class Series(SeriesCompat, BasePandasDataset):
             or return_type not in ["DataFrame", "Series"]
         ):
             # use the explicit non-Compat parent to avoid infinite recursion
-            result = BasePandasDataset._apply(
-                self,
+            result = super(Series, self).apply(
                 func,
                 axis=0,
                 broadcast=None,
@@ -687,7 +710,7 @@ class Series(SeriesCompat, BasePandasDataset):
 
             result = DataFrame(query_compiler=result)
         elif return_type == "Series":
-            result = Series(query_compiler=result)
+            result = self.__constructor__(query_compiler=result)
             if result.name == self.index[0]:
                 result.name = None
         elif isinstance(result, type(self._query_compiler)):
@@ -728,7 +751,7 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         return self.corr(self.shift(lag))
 
-    def _between(self, left, right, inclusive):  # noqa: PR01, RT01, D200
+    def between(self, left, right, inclusive: str = "both"):  # noqa: PR01, RT01, D200
         """
         Return boolean Series equivalent to left <= series <= right.
         """
@@ -744,20 +767,20 @@ class Series(SeriesCompat, BasePandasDataset):
             other, lambda s1, s2: s1.combine(s2, func, fill_value=fill_value)
         )
 
-    def _compare(
+    def compare(
         self,
-        other: "Series",
-        align_axis: Union[str, int],
-        keep_shape: bool,
-        keep_equal: bool,
-        result_names: tuple,
+        other: Series,
+        align_axis: Union[str, int] = 1,
+        keep_shape: bool = False,
+        keep_equal: bool = False,
+        result_names: tuple = ("self", "other"),
     ) -> "Series":  # noqa: PR01, RT01, D200
         """
         Compare to another Series and show the differences.
         """
         if not isinstance(other, Series):
             raise TypeError(f"Cannot compare Series to {type(other)}")
-        result = self.to_frame()._compare(
+        result = self.to_frame().compare(
             other.to_frame(),
             align_axis=align_axis,
             keep_shape=keep_shape,
@@ -979,12 +1002,17 @@ class Series(SeriesCompat, BasePandasDataset):
             ignore_index=ignore_index,
         )
 
-    def _factorize(self, sort, na_sentinel, **kwargs):  # noqa: PR01, RT01, D200
+    def factorize(
+        self, sort=False, na_sentinel=no_default, use_na_sentinel=no_default
+    ):  # noqa: PR01, RT01, D200
         """
         Encode the object as an enumerated type or categorical variable.
         """
         return self._default_to_pandas(
-            pandas.Series.factorize, sort=sort, na_sentinel=na_sentinel, **kwargs
+            pandas.Series.factorize,
+            sort=sort,
+            na_sentinel=na_sentinel,
+            use_na_sentinel=use_na_sentinel,
         )
 
     def fillna(
@@ -1004,7 +1032,7 @@ class Series(SeriesCompat, BasePandasDataset):
                 '"value" parameter must be a scalar, dict or Series, but '
                 + f'you passed a "{type(value).__name__}"'
             )
-        return super(Series, self)._fillna(
+        return super(Series, self).fillna(
             squeeze_self=True,
             squeeze_value=isinstance(value, Series),
             value=value,
@@ -1033,17 +1061,17 @@ class Series(SeriesCompat, BasePandasDataset):
         new_self, new_other = self._prepare_inter_op(other)
         return super(Series, new_self).ge(new_other, level=level, axis=axis)
 
-    def _groupby(
+    def groupby(
         self,
-        by,
-        axis,
-        level,
-        as_index,
-        sort,
-        group_keys,
-        squeeze,
-        observed,
-        dropna,
+        by=None,
+        axis=0,
+        level=None,
+        as_index=True,
+        sort=True,
+        group_keys=no_default,
+        squeeze=no_default,
+        observed=False,
+        dropna: bool = True,
     ):  # noqa: PR01, RT01, D200
         """
         Group Series using a mapper or by a Series of columns.
@@ -1123,21 +1151,38 @@ class Series(SeriesCompat, BasePandasDataset):
             **kwds,
         )
 
-    def _idxmax(self, axis=0, skipna=True, *args, **kwargs):  # noqa: PR01, RT01, D200
+    def idxmax(self, axis=0, skipna=True, *args, **kwargs):  # noqa: PR01, RT01, D200
         """
         Return the row label of the maximum value.
         """
         if skipna is None:
             skipna = True
-        return super(Series, self)._idxmax(axis=axis, skipna=skipna, *args, **kwargs)
+        return super(Series, self).idxmax(axis=axis, skipna=skipna, *args, **kwargs)
 
-    def _idxmin(self, axis=0, skipna=True, *args, **kwargs):  # noqa: PR01, RT01, D200
+    def idxmin(self, axis=0, skipna=True, *args, **kwargs):  # noqa: PR01, RT01, D200
         """
         Return the row label of the minimum value.
         """
         if skipna is None:
             skipna = True
-        return super(Series, self)._idxmin(axis=axis, skipna=skipna, *args, **kwargs)
+        return super(Series, self).idxmin(axis=axis, skipna=skipna, *args, **kwargs)
+
+    def info(
+        self,
+        verbose: bool | None = None,
+        buf: IO[str] | None = None,
+        max_cols: int | None = None,
+        memory_usage: bool | str | None = None,
+        show_counts: bool = True,
+    ):
+        return self._default_to_pandas(
+            pandas.Series.info,
+            verbose=verbose,
+            buf=buf,
+            max_cols=max_cols,
+            memory_usage=memory_usage,
+            show_counts=show_counts,
+        )
 
     def interpolate(
         self,
@@ -1165,6 +1210,12 @@ class Series(SeriesCompat, BasePandasDataset):
             **kwargs,
         )
 
+    def isin(self, values):  # noqa: PR01, RT01, D200
+        """
+        Whether elements in `Series` are contained in `values`.
+        """
+        return super(Series, self).isin(values, shape_hint="column")
+
     def item(self):  # noqa: RT01, D200
         """
         Return the first element of the underlying data as a Python scalar.
@@ -1195,21 +1246,21 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         return self.index
 
-    def _kurt(
+    def kurt(
         self,
-        axis,
-        skipna,
-        level,
-        numeric_only,
+        axis: Axis | None | NoDefault = no_default,
+        skipna=True,
+        level=None,
+        numeric_only=None,
         **kwargs,
     ):  # noqa: PR01, RT01, D200
         """
         Return unbiased kurtosis over requested axis.
         """
         axis = self._get_axis_number(axis)
-        return super(Series, self)._kurt(axis, skipna, level, numeric_only, **kwargs)
+        return super(Series, self).kurt(axis, skipna, level, numeric_only, **kwargs)
 
-    kurtosis = SeriesCompat.kurt
+    kurtosis = kurt
 
     def le(self, other, level=None, fill_value=None, axis=0):  # noqa: PR01, RT01, D200
         """
@@ -1229,6 +1280,14 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Map values of Series according to input correspondence.
         """
+        if isinstance(arg, type(self)):
+            # HACK: if we don't cast to pandas, then the execution engine will try to
+            # propagate the distributed Series to workers and most likely would have
+            # some performance problems.
+            # TODO: A better way of doing so could be passing this `arg` as a query compiler
+            # and broadcast accordingly.
+            arg = arg._to_pandas()
+
         if not callable(arg) and hasattr(arg, "get"):
             mapper = arg
 
@@ -1244,15 +1303,15 @@ class Series(SeriesCompat, BasePandasDataset):
         )
 
     @_inherit_docstrings(pandas.Series.mask, apilink="pandas.Series.mask")
-    def _mask(
+    def mask(
         self,
         cond,
-        other,
-        inplace,
-        axis,
-        level,
-        errors,
-        try_cast,
+        other=np.nan,
+        inplace=False,
+        axis=None,
+        level=None,
+        errors=no_default,
+        try_cast=no_default,
     ):
         return self._default_to_pandas(
             pandas.Series.mask,
@@ -1301,7 +1360,18 @@ class Series(SeriesCompat, BasePandasDataset):
             new_other, level=level, fill_value=None, axis=axis
         )
 
-    multiply = rmul = mul
+    multiply = mul
+
+    def rmul(
+        self, other, level=None, fill_value=None, axis=0
+    ):  # noqa: PR01, RT01, D200
+        """
+        Return multiplication of series and `other`, element-wise (binary operator `mul`).
+        """
+        new_self, new_other = self._prepare_inter_op(other)
+        return super(Series, new_self).rmul(
+            new_other, level=level, fill_value=None, axis=axis
+        )
 
     def ne(self, other, level=None, fill_value=None, axis=0):  # noqa: PR01, RT01, D200
         """
@@ -1320,7 +1390,9 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Return the smallest `n` elements.
         """
-        return Series(query_compiler=self._query_compiler.nsmallest(n=n, keep=keep))
+        return self.__constructor__(
+            query_compiler=self._query_compiler.nsmallest(n=n, keep=keep)
+        )
 
     def slice_shift(self, periods=1, axis=0):  # noqa: PR01, RT01, D200
         """
@@ -1331,7 +1403,7 @@ class Series(SeriesCompat, BasePandasDataset):
 
         if axis == "index" or axis == 0:
             if abs(periods) >= len(self.index):
-                return Series(dtype=self.dtype)
+                return self.__constructor__(dtype=self.dtype)
             else:
                 new_df = self.iloc[:-periods] if periods > 0 else self.iloc[-periods:]
                 new_df.index = (
@@ -1410,15 +1482,16 @@ class Series(SeriesCompat, BasePandasDataset):
         )
 
     @_inherit_docstrings(pandas.Series.prod, apilink="pandas.Series.prod")
-    def _prod(
+    def prod(
         self,
-        axis,
-        skipna,
-        level,
-        numeric_only,
-        min_count,
+        axis=None,
+        skipna=True,
+        level=None,
+        numeric_only=None,
+        min_count=0,
         **kwargs,
     ):
+        validate_bool_kwarg(skipna, "skipna", none_allowed=False)
         axis = self._get_axis_number(axis)
         if level is not None:
             if (
@@ -1458,7 +1531,7 @@ class Series(SeriesCompat, BasePandasDataset):
             )
         )
 
-    product = SeriesCompat.prod
+    product = prod
 
     def ravel(self, order="C"):  # noqa: PR01, RT01, D200
         """
@@ -1471,7 +1544,7 @@ class Series(SeriesCompat, BasePandasDataset):
         return data
 
     @_inherit_docstrings(pandas.Series.reindex, apilink="pandas.Series.reindex")
-    def _reindex(self, *args, **kwargs):
+    def reindex(self, *args, **kwargs):
         if args:
             if len(args) > 1:
                 raise TypeError("Only one positional argument ('index') is allowed")
@@ -1492,7 +1565,7 @@ class Series(SeriesCompat, BasePandasDataset):
                 "reindex() got an unexpected keyword "
                 + f'argument "{list(kwargs.keys())[0]}"'
             )
-        return super(Series, self)._reindex(
+        return super(Series, self).reindex(
             index=index,
             columns=None,
             method=method,
@@ -1544,8 +1617,13 @@ class Series(SeriesCompat, BasePandasDataset):
 
         return self.__constructor__(query_compiler=self._query_compiler.repeat(repeats))
 
-    def _reset_index(
-        self, level, drop, name, inplace, allow_duplicates
+    def reset_index(
+        self,
+        level=None,
+        drop=False,
+        name=no_default,
+        inplace=False,
+        allow_duplicates=False,
     ):  # noqa: PR01, RT01, D200
         """
         Generate a new Series with the index reset.
@@ -1572,7 +1650,7 @@ class Series(SeriesCompat, BasePandasDataset):
             obj.name = name
             from .dataframe import DataFrame
 
-            return DataFrame(obj)._reset_index(
+            return DataFrame(obj).reset_index(
                 level=level,
                 drop=drop,
                 inplace=inplace,
@@ -1653,11 +1731,7 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Return value at the given quantile.
         """
-        # We cannot use super().quantile() here as it's failing into infinite recursion
-        # because SeriesCompat is selected as super-class in this case due to how MRO works.
-        # We cannot change the order because it would break in other places by picking methods
-        # from BasePandasDataset instead of special-crafted SeriesCompat.
-        return self._quantile(
+        return super(Series, self).quantile(
             q=q,
             axis=0,
             numeric_only=False,
@@ -1671,14 +1745,14 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         return super(Series, self).reorder_levels(order)
 
-    def _replace(
+    def replace(
         self,
-        to_replace,
-        value,
-        inplace,
-        limit,
-        regex,
-        method,
+        to_replace=None,
+        value=no_default,
+        inplace=False,
+        limit=None,
+        regex=False,
+        method: str | NoDefault = no_default,
     ):  # noqa: PR01, RT01, D200
         """
         Replace values given in `to_replace` with `value`.
@@ -1788,18 +1862,19 @@ class Series(SeriesCompat, BasePandasDataset):
 
     subtract = sub
 
-    def _sum(
+    def sum(
         self,
-        axis,
-        skipna,
-        level,
-        numeric_only,
-        min_count,
+        axis=None,
+        skipna=True,
+        level=None,
+        numeric_only=None,
+        min_count=0,
         **kwargs,
     ):  # noqa: PR01, RT01, D200
         """
         Return the sum of the values.
         """
+        validate_bool_kwarg(skipna, "skipna", none_allowed=False)
         axis = self._get_axis_number(axis)
         if level is not None:
             if (
@@ -1860,7 +1935,9 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         return self._default_to_pandas("to_dict", into=into)
 
-    def _to_frame(self, name: "Hashable") -> "DataFrame":  # noqa: PR01, RT01, D200
+    def to_frame(
+        self, name: Hashable = no_default
+    ) -> "DataFrame":  # noqa: PR01, RT01, D200
         """
         Convert Series to {label -> value} dict or dict-like object.
         """
@@ -1887,15 +1964,22 @@ class Series(SeriesCompat, BasePandasDataset):
         """
         Return the NumPy ndarray representing the values in this Series or Index.
         """
-        return (
-            super(Series, self)
-            .to_numpy(
-                dtype=dtype,
-                copy=copy,
-                na_value=na_value,
+        from modin.config import ExperimentalNumPyAPI
+
+        if not ExperimentalNumPyAPI.get():
+            return (
+                super(Series, self)
+                .to_numpy(
+                    dtype=dtype,
+                    copy=copy,
+                    na_value=na_value,
+                )
+                .flatten()
             )
-            .flatten()
-        )
+        else:
+            from ..numpy.arr import array
+
+            return array(self, copy=copy)
 
     tolist = to_list
 
@@ -1988,12 +2072,12 @@ class Series(SeriesCompat, BasePandasDataset):
         Modify Series in place using values from passed Series.
         """
         if not isinstance(other, Series):
-            other = Series(other)
+            other = self.__constructor__(other)
         query_compiler = self._query_compiler.series_update(other._query_compiler)
         self._update_inplace(new_query_compiler=query_compiler)
 
-    def _value_counts(
-        self, normalize, sort, ascending, bins, dropna
+    def value_counts(
+        self, normalize=False, sort=True, ascending=False, bins=None, dropna=True
     ):  # noqa: PR01, RT01, D200
         """
         Return a Series containing counts of unique values.
@@ -2001,7 +2085,7 @@ class Series(SeriesCompat, BasePandasDataset):
         if bins is not None:
             # Potentially we could implement `cut` function from pandas API, which
             # bins values into intervals, and then we can just count them as regular values.
-            # TODO #1333: new_self = Series(pd.cut(self, bins, include_lowest=True), dtype="interval")
+            # TODO #1333: new_self = self.__constructor__(pd.cut(self, bins, include_lowest=True), dtype="interval")
             return self._default_to_pandas(
                 pandas.Series.value_counts,
                 normalize=normalize,
@@ -2010,7 +2094,7 @@ class Series(SeriesCompat, BasePandasDataset):
                 bins=bins,
                 dropna=dropna,
             )
-        counted_values = super(Series, self)._value_counts(
+        counted_values = super(Series, self).value_counts(
             subset=self,
             normalize=normalize,
             sort=sort,
@@ -2029,15 +2113,15 @@ class Series(SeriesCompat, BasePandasDataset):
             query_compiler=self._query_compiler.series_view(dtype=dtype)
         )
 
-    def _where(
+    def where(
         self,
         cond,
-        other,
-        inplace,
-        axis,
-        level,
-        errors,
-        try_cast,
+        other=no_default,
+        inplace=False,
+        axis=None,
+        level=None,
+        errors=no_default,
+        try_cast=no_default,
     ):  # noqa: PR01, RT01, D200
         """
         Replace values where the condition is False.
@@ -2055,14 +2139,6 @@ class Series(SeriesCompat, BasePandasDataset):
             try_cast=try_cast,
         )
 
-    def xs(
-        self, key, axis=0, level=None, drop_level=True
-    ):  # pragma: no cover # noqa: PR01, D200
-        """
-        Return cross-section from the Series/DataFrame.
-        """
-        raise NotImplementedError("Not Yet implemented.")
-
     @property
     def attrs(self):  # noqa: RT01, D200
         """
@@ -2244,6 +2320,12 @@ class Series(SeriesCompat, BasePandasDataset):
             query_compiler=self._query_compiler.to_numeric(**kwargs)
         )
 
+    def _qcut(self, q, **kwargs):  # noqa: PR01, RT01, D200
+        """
+        Quantile-based discretization function.
+        """
+        return self._default_to_pandas(pandas.qcut, q, **kwargs)
+
     def _reduce_dimension(self, query_compiler):
         """
         Try to reduce the dimension of data from the `query_compiler`.
@@ -2383,7 +2465,7 @@ class Series(SeriesCompat, BasePandasDataset):
             or type(new_query_compiler) in self._query_compiler.__class__.__bases__
         ), "Invalid Query Compiler object: {}".format(type(new_query_compiler))
         if not inplace and new_query_compiler.is_series_like():
-            return Series(query_compiler=new_query_compiler)
+            return self.__constructor__(query_compiler=new_query_compiler)
         elif not inplace:
             # This can happen with things like `reset_index` where we can add columns.
             from .dataframe import DataFrame
@@ -2409,12 +2491,15 @@ class Series(SeriesCompat, BasePandasDataset):
             Prepared `other`.
         """
         if isinstance(other, Series):
-            # NB: deep=False is important for performance bc it retains obj.index._id
-            new_self = self.copy(deep=False)
-            new_other = other.copy(deep=False)
-            if self.name == other.name:
-                new_self.name = new_other.name = self.name
-            else:
+            names_different = self.name != other.name
+            # NB: if we don't need a rename, do the interaction with shallow
+            # copies so that we preserve obj.index._id. It's fine to work
+            # with shallow copies because we'll discard the copies but keep
+            # the result after the interaction opreation. We can't do a rename
+            # on shallow copies because we'll mutate the original objects.
+            new_self = self.copy(deep=names_different)
+            new_other = other.copy(deep=names_different)
+            if names_different:
                 new_self.name = new_other.name = MODIN_UNNAMED_SERIES_LABEL
         else:
             new_self = self
@@ -2471,6 +2556,20 @@ class Series(SeriesCompat, BasePandasDataset):
             return self._reduce_dimension(result)
         return self.__constructor__(query_compiler=result)
 
+    def _repartition(self):
+        """
+        Repartitioning Series to get ideal partitions inside.
+
+        Allows to improve performance where the query compiler can't improve
+        yet by doing implicit repartitioning.
+
+        Returns
+        -------
+        Series
+            The repartitioned Series.
+        """
+        return super()._repartition(axis=0)
+
     # Persistance support methods - BEGIN
     @classmethod
     def _inflate_light(cls, query_compiler, name):
