@@ -296,6 +296,14 @@ class StringMethods(ClassLogger):
         )
 
     def extract(self, pat, flags=0, expand=True):
+        if expand:
+            from .dataframe import DataFrame
+
+            return DataFrame(
+                query_compiler=self._query_compiler.str_extract(
+                    pat, flags=flags, expand=expand
+                )
+            )
         return self._default_to_pandas(
             pandas.Series.str.extract, pat, flags=flags, expand=expand
         )
@@ -331,7 +339,7 @@ class StringMethods(ClassLogger):
             )
 
     def repeat(self, repeats):
-        return self._default_to_pandas(pandas.Series.str.repeat, repeats)
+        return Series(query_compiler=self._query_compiler.str_repeat(repeats))
 
     def rpartition(self, sep=" ", expand=True):
         if sep is not None and len(sep) == 0:
