@@ -19,6 +19,7 @@ import numpy as np
 from numpy.testing import assert_array_equal
 from modin.utils import get_current_execution, to_pandas
 from modin.test.test_utils import warns_that_defaulting_to_pandas
+from modin.config import Engine
 from pandas.testing import assert_frame_equal
 
 from .utils import (
@@ -28,6 +29,8 @@ from .utils import (
     df_equals,
     sort_index_for_equal_values,
     eval_general,
+    bool_arg_values,
+    bool_arg_keys,
 )
 
 
@@ -308,7 +311,7 @@ def test_merge_asof_suffixes():
             suffixes=(False, False),
         )
     with pytest.raises(ValueError), warns_that_defaulting_to_pandas():
-        modin_merged = pd.merge_asof(
+        pd.merge_asof(
             modin_left,
             modin_right,
             left_index=True,
@@ -636,7 +639,11 @@ def test_unique():
 def test_value_counts(normalize, bins, dropna):
     # We sort indices for Modin and pandas result because of issue #1650
     values = np.array([3, 1, 2, 3, 4, np.nan])
-    with warns_that_defaulting_to_pandas():
+    with (
+        _nullcontext()
+        if Engine.get() in ["Ray", "Dask", "Unidist"]
+        else warns_that_defaulting_to_pandas()
+    ):
         modin_result = sort_index_for_equal_values(
             pd.value_counts(values, normalize=normalize, ascending=False), False
         )
@@ -654,7 +661,11 @@ def test_value_counts(normalize, bins, dropna):
     )
     df_equals(modin_result, pandas_result)
 
-    with warns_that_defaulting_to_pandas():
+    with (
+        _nullcontext()
+        if Engine.get() in ["Ray", "Dask", "Unidist"]
+        else warns_that_defaulting_to_pandas()
+    ):
         modin_result = sort_index_for_equal_values(
             pd.value_counts(values, dropna=dropna, ascending=True), True
         )
@@ -716,6 +727,28 @@ def test_to_numeric(data, errors, downcast):
     df_equals(modin_result, pandas_result)
 
 
+@pytest.mark.parametrize("retbins", bool_arg_values, ids=bool_arg_keys)
+def test_qcut(retbins):
+    # test case from https://github.com/modin-project/modin/issues/5610
+    pandas_series = pandas.Series(range(10))
+    modin_series = pd.Series(range(10))
+    pandas_result = pandas.qcut(pandas_series, 4, retbins=retbins)
+    with warns_that_defaulting_to_pandas():
+        modin_result = pd.qcut(modin_series, 4, retbins=retbins)
+    if retbins:
+        df_equals(modin_result[0], pandas_result[0])
+        df_equals(modin_result[0].cat.categories, pandas_result[0].cat.categories)
+        assert_array_equal(modin_result[1], pandas_result[1])
+    else:
+        df_equals(modin_result, pandas_result)
+        df_equals(modin_result.cat.categories, pandas_result.cat.categories)
+
+    # test case for fallback to pandas, taken from pandas docs
+    pandas_result = pandas.qcut(range(5), 4)
+    modin_result = pd.qcut(range(5), 4)
+    df_equals(modin_result, pandas_result)
+
+
 @pytest.mark.parametrize(
     "data", [test_data_values[0], []], ids=["test_data_values[0]", "[]"]
 )
