@@ -36,7 +36,6 @@ from modin.config import (
     TestReadFromSqlServer,
     ReadSqlEngine,
 )
-from modin._compat import PandasCompatVersion
 from modin.utils import to_pandas
 from modin.pandas.utils import from_arrow
 from modin.test.test_utils import warns_that_defaulting_to_pandas
@@ -163,11 +162,13 @@ def parquet_eval_to_file(modin_obj, pandas_obj, fn, extension, **fn_kwargs):
             extension=extension, data_dir=dirname
         )
 
+        engine = fn_kwargs.get("engine", "auto")
+
         getattr(modin_obj, fn)(unique_filename_modin, **fn_kwargs)
         getattr(pandas_obj, fn)(unique_filename_pandas, **fn_kwargs)
 
-        pandas_df = pandas.read_parquet(unique_filename_pandas)
-        modin_df = pd.read_parquet(unique_filename_modin)
+        pandas_df = pandas.read_parquet(unique_filename_pandas, engine=engine)
+        modin_df = pd.read_parquet(unique_filename_modin, engine=engine)
     df_equals(pandas_df, modin_df)
 
 
@@ -199,7 +200,8 @@ def eval_to_file(modin_obj, pandas_obj, fn, extension, **fn_kwargs):
                 last_exception = err
                 continue
             break
-        else:
+        # If we do have an exception that's valid let's raise it
+        if last_exception:
             raise last_exception
 
         getattr(pandas_obj, fn)(unique_filename_pandas, **fn_kwargs)
@@ -322,7 +324,6 @@ class TestCsv:
         converters,
         skipfooter,
     ):
-
         if dtype:
             dtype = {
                 col: "object"
@@ -333,7 +334,6 @@ class TestCsv:
 
         eval_io(
             fn_name="read_csv",
-            check_exception_type=None,  # issue #2320
             raising_exceptions=None,
             check_kwargs_callable=not callable(converters),
             # read_csv kwargs
@@ -376,17 +376,6 @@ class TestCsv:
         names,
         encoding,
     ):
-        xfail_case = (
-            StorageFormat.get() == "Hdk"
-            and header is not None
-            and isinstance(skiprows, int)
-            and names is None
-            and nrows is None
-        )
-        if xfail_case:
-            pytest.xfail(
-                "read_csv fails because of duplicated columns names - issue #3080"
-            )
         if request.config.getoption(
             "--simulate-cloud"
         ).lower() != "off" and is_list_like(skiprows):
@@ -417,9 +406,9 @@ class TestCsv:
                     pytest.xfail(
                         "read_csv incorrect output with float data - issue #2634"
                     )
+
             eval_io(
                 fn_name="read_csv",
-                check_exception_type=None,  # issue #2320
                 raising_exceptions=None,
                 check_kwargs_callable=not callable(skiprows),
                 # read_csv kwargs
@@ -447,7 +436,6 @@ class TestCsv:
 
         eval_io(
             fn_name="read_csv",
-            check_exception_type=None,  # issue #2320
             raising_exceptions=None,
             # read_csv kwargs
             filepath_or_buffer=pytest.csvs_names["test_read_csv_yes_no"],
@@ -493,16 +481,8 @@ class TestCsv:
             )
 
     def test_read_csv_mangle_dupe_cols(self):
-        if StorageFormat.get() == "Hdk":
-            pytest.xfail(
-                "processing of duplicated columns in HDK storage format is not supported yet - issue #3080"
-            )
-        with ensure_clean() as unique_filename, (
-            pytest.warns(
-                FutureWarning, match="'mangle_dupe_cols' keyword is deprecated"
-            )
-            if PandasCompatVersion.CURRENT == PandasCompatVersion.LATEST
-            else _nullcontext()
+        with ensure_clean() as unique_filename, pytest.warns(
+            FutureWarning, match="'mangle_dupe_cols' keyword is deprecated"
         ):
             str_non_unique_cols = "col,col,col,col\n5, 6, 7, 8\n9, 10, 11, 12\n"
             eval_io_from_str(
@@ -648,13 +628,6 @@ class TestCsv:
     @pytest.mark.parametrize("encoding", [None, "latin8", "utf16"])
     @pytest.mark.parametrize("engine", [None, "python", "c"])
     def test_read_csv_compression(self, make_csv_file, compression, encoding, engine):
-        if (
-            compression == "zip"
-            and PandasCompatVersion.CURRENT == PandasCompatVersion.PY36
-        ):
-            pytest.xfail(
-                "Parallel read_csv does not support compression with older pandas"
-            )
         with ensure_clean(".csv") as unique_filename:
             make_csv_file(
                 filename=unique_filename, encoding=encoding, compression=compression
@@ -714,7 +687,7 @@ class TestCsv:
     @pytest.mark.parametrize("decimal", [".", "_"])
     @pytest.mark.parametrize("lineterminator", [None, "x", "\n"])
     @pytest.mark.parametrize("escapechar", [None, "d", "x"])
-    @pytest.mark.parametrize("dialect", ["test_csv_dialect", None])
+    @pytest.mark.parametrize("dialect", ["test_csv_dialect", "use_dialect_name", None])
     def test_read_csv_file_format(
         self,
         make_csv_file,
@@ -724,17 +697,6 @@ class TestCsv:
         escapechar,
         dialect,
     ):
-        if Engine.get() != "Python" and lineterminator == "x":
-            pytest.xfail("read_csv with Ray engine outputs empty frame - issue #2493")
-        elif Engine.get() != "Python" and escapechar:
-            pytest.xfail(
-                "read_csv with Ray engine fails with some 'escapechar' parameters - issue #2494"
-            )
-        elif Engine.get() != "Python" and dialect:
-            pytest.xfail(
-                "read_csv with Ray engine fails with `dialect` parameter - issue #2508"
-            )
-
         with ensure_clean(".csv") as unique_filename:
             if dialect:
                 test_csv_dialect_params = {
@@ -745,7 +707,9 @@ class TestCsv:
                     "quoting": csv.QUOTE_ALL,
                 }
                 csv.register_dialect(dialect, **test_csv_dialect_params)
-                dialect = csv.get_dialect(dialect)
+                if dialect != "use_dialect_name":
+                    # otherwise try with dialect name instead of `_csv.Dialect` object
+                    dialect = csv.get_dialect(dialect)
                 make_csv_file(filename=unique_filename, **test_csv_dialect_params)
             else:
                 make_csv_file(
@@ -756,8 +720,22 @@ class TestCsv:
                     line_terminator=lineterminator,
                 )
 
+            if (
+                (StorageFormat.get() == "Hdk")
+                and (escapechar is not None)
+                and (lineterminator is None)
+                and (thousands is None)
+                and (decimal == ".")
+            ):
+                with open(unique_filename, "r") as f:
+                    if any(
+                        line.find(f',"{escapechar}') != -1 for _, line in enumerate(f)
+                    ):
+                        pytest.xfail(
+                            "Tests with this character sequence fail due to #5649"
+                        )
+
             eval_io(
-                check_exception_type=None,  # issue #2320
                 raising_exceptions=None,
                 fn_name="read_csv",
                 # read_csv kwargs
@@ -815,10 +793,6 @@ class TestCsv:
     @pytest.mark.parametrize("warn_bad_lines", [True, False, None])
     @pytest.mark.parametrize("error_bad_lines", [True, False, None])
     @pytest.mark.parametrize("on_bad_lines", ["error", "warn", "skip", None])
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="In compat mode, some error handling tests are failing due to https://github.com/modin-project/modin/issues/2845",
-    )
     def test_read_csv_error_handling(
         self,
         warn_bad_lines,
@@ -842,11 +816,7 @@ class TestCsv:
             filepath_or_buffer=pytest.csvs_names["test_read_csv_bad_lines"],
             warn_bad_lines=warn_bad_lines,
             error_bad_lines=error_bad_lines,
-            **(
-                {}
-                if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36
-                else dict(on_bad_lines=on_bad_lines)
-            ),
+            on_bad_lines=on_bad_lines,
         )
 
     # Internal parameters tests
@@ -988,17 +958,9 @@ class TestCsv:
             index_col=index_col,
             parse_dates=parse_dates,
             encoding=encoding,
-            **(
-                {}
-                if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36
-                else dict(encoding_errors=encoding_errors)
-            ),
+            encoding_errors=encoding_errors,
         )
 
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="storage_options not supported for older pandas",
-    )
     @pytest.mark.parametrize(
         "storage_options",
         [{"anon": False}, {"anon": True}, {"key": "123", "secret": "123"}, None],
@@ -1014,10 +976,6 @@ class TestCsv:
             storage_options=storage_options,
         )
 
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="storage_options not supported for older pandas",
-    )
     def test_read_csv_s3_issue4658(self):
         eval_io(
             fn_name="read_csv",
@@ -1030,13 +988,6 @@ class TestCsv:
     @pytest.mark.parametrize("names", [list("XYZ"), None])
     @pytest.mark.parametrize("skiprows", [1, 2, 3, 4, None])
     def test_read_csv_skiprows_names(self, names, skiprows):
-        if StorageFormat.get() == "Hdk" and names is None and skiprows in [1, None]:
-            # If these conditions are satisfied, columns names will be inferred
-            # from the first row, that will contain duplicated values, that is
-            # not supported by  `HDK` storage format yet.
-            pytest.xfail(
-                "processing of duplicated columns in HDK storage format is not supported yet - issue #3080"
-            )
         eval_io(
             fn_name="read_csv",
             # read_csv kwargs
@@ -1063,9 +1014,8 @@ class TestCsv:
             # This tests that we default to pandas on a buffer
             from io import StringIO
 
-            pd.read_csv(
-                StringIO(open(pytest.csvs_names["test_read_csv_regular"], "r").read())
-            )
+            with open(pytest.csvs_names["test_read_csv_regular"], "r") as _f:
+                pd.read_csv(StringIO(_f.read()))
 
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
@@ -1144,7 +1094,6 @@ class TestCsv:
         )
 
     def test_read_csv_wrong_path(self):
-
         raising_exceptions = [e for e in io_ops_bad_exc if e != FileNotFoundError]
 
         eval_io(
@@ -1165,9 +1114,6 @@ class TestCsv:
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #2340",
     )
     def test_to_csv(self, header, mode):
-        if "b" in mode and PandasCompatVersion.CURRENT == PandasCompatVersion.PY36:
-            pytest.xfail(reason="older pandas does not support to_csv with binary mode")
-
         pandas_df = generate_dataframe()
         modin_df = pd.DataFrame(pandas_df)
 
@@ -1334,6 +1280,17 @@ class TestCsv:
         ).set_index("key")
         eval_to_file(modin_df, pandas_df, "to_csv", "csv")
 
+    def test_read_csv_issue_5150(self):
+        with ensure_clean(".csv") as unique_filename:
+            pandas_df = pandas.DataFrame(
+                np.random.randint(0, 100, size=(2**6, 2**6))
+            )
+            pandas_df.to_csv(unique_filename, index=False)
+            expected_pandas_df = pandas.read_csv(unique_filename, index_col=False)
+            modin_df = pd.read_csv(unique_filename, index_col=False)
+            actual_pandas_df = modin_df._to_pandas()
+        df_equals(expected_pandas_df, actual_pandas_df)
+
 
 class TestTable:
     def test_read_table(self, make_csv_file):
@@ -1375,17 +1332,17 @@ class TestTable:
             )
 
 
+@pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
 class TestParquet:
     @pytest.mark.parametrize("columns", [None, ["col1"]])
     @pytest.mark.parametrize("row_group_size", [None, 100, 1000, 10_000])
-    @pytest.mark.parametrize("engine", ["auto", "pyarrow", "fastparquet"])
     @pytest.mark.parametrize("path_type", [Path, str])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
     def test_read_parquet(
-        self, make_parquet_file, columns, row_group_size, engine, path_type
+        self, engine, make_parquet_file, columns, row_group_size, path_type
     ):
         with ensure_clean(".parquet") as unique_filename:
             unique_filename = path_type(unique_filename)
@@ -1399,11 +1356,21 @@ class TestParquet:
                 columns=columns,
             )
 
+    def test_read_parquet_list_of_files_5698(self, engine, make_parquet_file):
+        if engine == "fastparquet" and os.name == "nt":
+            pytest.xfail(reason="https://github.com/pandas-dev/pandas/issues/51720")
+        with ensure_clean(".parquet") as f1, ensure_clean(
+            ".parquet"
+        ) as f2, ensure_clean(".parquet") as f3:
+            for f in [f1, f2, f3]:
+                make_parquet_file(filename=f)
+            eval_io(fn_name="read_parquet", path=[f1, f2, f3], engine=engine)
+
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
-    def test_read_parquet_indexing_by_column(self, make_parquet_file):
+    def test_read_parquet_indexing_by_column(self, engine, make_parquet_file):
         # Test indexing into a column of Modin with various parquet file row lengths.
         # Specifically, tests for https://github.com/modin-project/modin/issues/3527
         # which fails when min_partition_size < nrows < min_partition_size * (num_partitions - 1)
@@ -1415,7 +1382,7 @@ class TestParquet:
             unique_filename = get_unique_filename(extension="parquet", data_dir=dirname)
             make_parquet_file(filename=unique_filename, nrows=nrows)
 
-            parquet_df = pd.read_parquet(unique_filename)
+            parquet_df = pd.read_parquet(unique_filename, engine=engine)
         for col in parquet_df.columns:
             parquet_df[col]
 
@@ -1424,13 +1391,12 @@ class TestParquet:
     @pytest.mark.parametrize(
         "rows_per_file", [[1000] * 40, [0, 0, 40_000], [10_000, 10_000] + [100] * 200]
     )
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
     def test_read_parquet_directory(
-        self, make_parquet_dir, columns, row_group_size, rows_per_file, engine
+        self, engine, make_parquet_dir, columns, row_group_size, rows_per_file
     ):
         num_cols = DATASET_SIZE_DICT.get(
             TestDatasetSize.get(), DATASET_SIZE_DICT["Small"]
@@ -1460,7 +1426,6 @@ class TestParquet:
             columns=columns,
         )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.parametrize("columns", [None, ["col1"]])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
@@ -1481,7 +1446,6 @@ class TestParquet:
                 columns=columns,
             )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1540,7 +1504,6 @@ class TestParquet:
                 engine=engine,
             )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1566,7 +1529,6 @@ class TestParquet:
                 engine=engine,
             )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1581,19 +1543,8 @@ class TestParquet:
 
     @pytest.mark.parametrize(
         "path_type",
-        [
-            "object",
-            "directory",
-            pytest.param(
-                "url",
-                marks=pytest.mark.xfail(
-                    PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-                    reason="older pandas.read_parquet does not support storage_options",
-                ),
-            ),
-        ],
+        ["object", "directory", "url"],
     )
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1607,18 +1558,6 @@ class TestParquet:
             with fs.open(dataset_url, "rb") as file_obj:
                 eval_io("read_parquet", path=file_obj, engine=engine)
         elif path_type == "directory":
-            if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36:
-                if engine == "pyarrow":
-                    pytest.xfail(
-                        reason="older pandas.read_parquet does not support storage_options"
-                    )
-                else:
-                    # older pandas.read_parquet does not support
-                    # storage_options, but modin and pandas both happen to
-                    # give different ValueErrors, so we cannot xfail.
-                    # n.b. conda doesn't seem to have pytest>6.2.5 for python
-                    # 3.6, and pytest 6.2.5 doesn't accept "skip" parameter.
-                    pytest.skip()
             eval_io(
                 "read_parquet",
                 path="s3://modin-datasets/test_data_dir.parquet",
@@ -1626,10 +1565,6 @@ class TestParquet:
                 engine=engine,
             )
         else:
-            if PandasCompatVersion.CURRENT == PandasCompatVersion.PY36:
-                pytest.xfail(
-                    reason="older pandas.read_parquet does not support storage_options"
-                )
             eval_io(
                 "read_parquet",
                 path=dataset_url,
@@ -1637,7 +1572,6 @@ class TestParquet:
                 engine=engine,
             )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1670,7 +1604,6 @@ class TestParquet:
                 engine=engine,
             )
 
-    @pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
     def test_read_empty_parquet_file(self, engine):
         test_df = pandas.DataFrame()
         with tempfile.TemporaryDirectory() as directory:
@@ -1683,30 +1616,57 @@ class TestParquet:
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
-    def test_to_parquet(self):
+    def test_to_parquet(self, engine):
         modin_df, pandas_df = create_test_dfs(TEST_DATA)
         parquet_eval_to_file(
             modin_obj=modin_df,
             pandas_obj=pandas_df,
             fn="to_parquet",
             extension="parquet",
+            engine=engine,
+        )
+
+    def test_to_parquet_keep_index(self, engine):
+        data = {"c0": [0, 1] * 1000, "c1": [2, 3] * 1000}
+        modin_df, pandas_df = create_test_dfs(data)
+        modin_df.index.name = "foo"
+        pandas_df.index.name = "foo"
+
+        parquet_eval_to_file(
+            modin_obj=modin_df,
+            pandas_obj=pandas_df,
+            fn="to_parquet",
+            extension="parquet",
+            index=True,
+            engine=engine,
         )
 
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
-    def test_read_parquet_2462(self):
+    def test_read_parquet_2462(self, engine):
         test_df = pandas.DataFrame({"col1": [["ad_1", "ad_2"], ["ad_3"]]})
 
         with tempfile.TemporaryDirectory() as directory:
             path = f"{directory}/data"
             os.makedirs(path)
-            test_df.to_parquet(path + "/part-00000.parquet")
-            read_df = pd.read_parquet(path)
+            test_df.to_parquet(path + "/part-00000.parquet", engine=engine)
+            read_df = pd.read_parquet(path, engine=engine)
 
             df_equals(test_df, read_df)
 
+    def test_read_parquet_s3_with_column_partitioning(self, engine):
+        # This test case comes from
+        # https://github.com/modin-project/modin/issues/4636
+        dataset_url = "s3://modin-datasets/modin-bugs/modin_bug_5159_parquet/df.parquet"
+        eval_io(
+            fn_name="read_parquet",
+            path=dataset_url,
+            engine=engine,
+            storage_options={"anon": True},
+        )
+
 
 class TestJson:
     @pytest.mark.parametrize("lines", [False, True])
@@ -1718,10 +1678,6 @@ class TestJson:
             lines=lines,
         )
 
-    @pytest.mark.skipif(
-        condition=PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_json does not support storage_options",
-    )
     @pytest.mark.parametrize(
         "storage_options",
         [{"anon": False}, {"anon": True}, {"key": "123", "secret": "123"}, None],
@@ -1843,10 +1799,6 @@ class TestExcel:
         )
 
     @check_file_leaks
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_excel cannot read .xslx",
-    )
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
@@ -1869,10 +1821,6 @@ class TestExcel:
         reason="pandas throws the exception. See pandas issue #39250 for more info",
     )
     @check_file_leaks
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_excel cannot read .xslx",
-    )
     def test_read_excel_sheetname_title(self):
         eval_io(
             fn_name="read_excel",
@@ -1881,10 +1829,6 @@ class TestExcel:
         )
 
     @check_file_leaks
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_excel cannot read .xslx",
-    )
     def test_excel_empty_line(self):
         path = "modin/pandas/test/data/test_emptyline.xlsx"
         modin_df = pd.read_excel(path)
@@ -1940,10 +1884,6 @@ class TestExcel:
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="TypeError: Expected list, got type - issue #3284",
     )
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_excel cannot read .xslx",
-    )
     def test_ExcelFile(self, make_excel_file):
         unique_filename = make_excel_file()
 
@@ -1977,11 +1917,6 @@ class TestExcel:
 
             assert assert_files_eq(unique_filename_modin, unique_filename_pandas)
 
-    @pytest.mark.xfail(
-        Engine.get() != "Python"
-        and PandasCompatVersion.CURRENT != PandasCompatVersion.PY36,
-        reason="Test fails because of issue 3305",
-    )
     @check_file_leaks
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
@@ -2017,7 +1952,6 @@ class TestHdf:
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
     def test_HDFStore(self):
-        hdf_file = None
         with ensure_clean_dir() as dirname:
             unique_filename_modin = get_unique_filename(
                 extension="hdf", data_dir=dirname
@@ -2334,13 +2268,6 @@ class TestFwf:
 
     @pytest.mark.parametrize("nrows", [13, None])
     def test_fwf_file_skiprows(self, make_fwf_file, nrows):
-        if (
-            nrows is not None
-            and PandasCompatVersion.CURRENT == PandasCompatVersion.PY36
-        ):
-            pytest.xfail(
-                "older pandas read_fwf had a bug with list of skiprows and non-None nrows: pandas-dev#10261"
-            )
         unique_filename = make_fwf_file()
 
         eval_io(
@@ -2488,6 +2415,15 @@ class TestStata:
         )
 
 
+class TestSas:
+    def test_read_sas(self):
+        eval_io(
+            fn_name="read_sas",
+            # read_stata kwargs
+            filepath_or_buffer="modin/pandas/test/data/airline.sas7bdat",
+        )
+
+
 class TestFeather:
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
@@ -2508,10 +2444,6 @@ class TestFeather:
         "storage_options",
         [{"anon": False}, {"anon": True}, {"key": "123", "secret": "123"}, None],
     )
-    @pytest.mark.skipif(
-        PandasCompatVersion.CURRENT == PandasCompatVersion.PY36,
-        reason="older pandas read_feather does not support storage_options",
-    )
     def test_read_feather_s3(self, storage_options):
         eval_io(
             fn_name="read_feather",
